@@ -12,16 +12,18 @@ from qtpy import QtCore, QtWidgets, QtGui, compat
 from qtpy.QtCore import Qt
 
 from glue.core.application_base import Application
-from glue.core.message import ApplicationClosedMessage, DataCollectionMessage
-from glue.core import command, Data
+from glue.core.message import ApplicationClosedMessage, DataCollectionMessage, SettingsChangeMessage
+from glue.core import command, BaseData
 from glue.core.coordinates import WCSCoordinates
 from glue import env
 from glue.main import load_plugins
 from glue.icons.qt import get_icon
-from glue.utils.qt import get_qapp
+from glue.utils.qt import get_qapp, update_global_font_size
 from glue.app.qt.actions import action
 from glue.dialogs.data_wizard.qt import data_wizard
 from glue.dialogs.link_editor.qt import LinkEditor
+from glue.dialogs.autolinker.qt import run_autolinker
+from glue.dialogs.component_arithmetic.qt import ArithmeticEditorWidget
 from glue.app.qt.edit_subset_mode_toolbar import EditSubsetModeToolBar
 from glue.app.qt.mdi_area import GlueMdiArea
 from glue.app.qt.layer_tree_widget import PlotAction, LayerTreeWidget
@@ -68,7 +70,7 @@ def _fix_ipython_pylab():
             pass
 
     try:
-        shell.enable_pylab('inline', import_all=True)
+        shell.enable_pylab('agg', import_all=True)
     except ValueError:
         # if the shell is a normal terminal shell, we get here
         pass
@@ -77,6 +79,9 @@ def _fix_ipython_pylab():
         pass
     except UsageError:
         pass
+    except KeyError:
+        # old versions of ipython
+        pass
 
     # Make sure we disable interactive mode (where figures get redrawn for
     # every single Matplotlib command)
@@ -133,15 +138,22 @@ class GlueLogger(QtWidgets.QWidget):
 
     def _set_console_button(self, attention):
         if attention:
-            self.button_console.setStyleSheet('color: red; text-decoration: underline;')
+            style = 'color: red; text-decoration: underline;'
         else:
-            self.button_console.setStyleSheet(self.button_stylesheet)
+            style = self.button_stylesheet
+        try:
+            self.button_console.setStyleSheet(style)
+        except RuntimeError:  # Prevent RuntimeError: wrapped C/C++ object of type QToolButton has been deleted
+            pass
 
     def write(self, message):
         """
         Interface for sys.excepthook
         """
-        self._stderr_original.write(message)
+        # On Windows, sys.stderr can sometimes be None, in which case we only
+        # show the warnings/errors in the graphical glue logger.
+        if self._stderr_original is not None:
+            self._stderr_original.write(message)
         self._text.moveCursor(QtGui.QTextCursor.End)
         self._text.insertPlainText(message)
         self._set_console_button(attention=True)
@@ -219,6 +231,34 @@ class ExportHelper(object):
             return saver(app, label)
 
 
+class ImportHelper(object):
+
+    def __init__(self, app):
+        self.app = weakref.ref(app)
+
+    def _choose_load_data_wizard(self, *args):
+        self._choose_load_data(data_importer=data_wizard)
+
+    def _choose_load_data(self, data_importer=None):
+
+        app = self.app()
+        if app is None:
+            return
+
+        if data_importer is None:
+            app.add_datasets(app.data_collection, data_wizard())
+        else:
+            data = data_importer()
+            if not isinstance(data, list):
+                raise TypeError("Data loader should return list of "
+                                "Data objects")
+            for item in data:
+                if not isinstance(item, BaseData):
+                    raise TypeError("Data loader should return list of "
+                                    "Data objects")
+            app.add_datasets(app.data_collection, data)
+
+
 class GlueApplication(Application, QtWidgets.QMainWindow):
 
     """ The main GUI application for the Qt frontend"""
@@ -234,6 +274,7 @@ class GlueApplication(Application, QtWidgets.QMainWindow):
             self._original_icon = self._original_app.windowIcon()
 
         self._export_helper = ExportHelper(self)
+        self._import_helper = ImportHelper(self)
 
         # Now we can get the application instance, which involves setting it
         # up if it doesn't already exist.
@@ -334,44 +375,10 @@ class GlueApplication(Application, QtWidgets.QMainWindow):
 
         # Data toolbar
 
-        self._data_toolbar = QtWidgets.QToolBar()
+        self._data_toolbar = QtWidgets.QToolBar("Session and Data toolbar")
 
         self._data_toolbar.setIconSize(QtCore.QSize(16, 16))
 
-        self._button_open_data = QtWidgets.QToolButton()
-        self._button_open_data.setText("Open Data")
-        self._button_open_data.setIcon(get_icon('glue_open'))
-        self._button_open_data.setToolButtonStyle(Qt.ToolButtonTextBesideIcon)
-        self._button_open_data.clicked.connect(self._choose_load_data_wizard)
-
-        self._data_toolbar.addWidget(self._button_open_data)
-
-        self._button_save_data = QtWidgets.QToolButton()
-        self._button_save_data.setText("Export Data/Subsets")
-        self._button_save_data.setIcon(get_icon('glue_filesave'))
-        self._button_save_data.setToolButtonStyle(Qt.ToolButtonTextBesideIcon)
-        self._button_save_data.clicked.connect(self._choose_save_data)
-
-        self._data_toolbar.addWidget(self._button_save_data)
-
-        self._button_link_data = QtWidgets.QToolButton()
-        self._button_link_data.setText("Link Data")
-        self._button_link_data.setIcon(get_icon('glue_link'))
-        self._button_link_data.setToolButtonStyle(Qt.ToolButtonTextBesideIcon)
-        self._button_link_data.clicked.connect(self._set_up_links)
-        self._on_data_collection_change()
-
-        self._data_toolbar.addWidget(self._button_link_data)
-
-        self._button_ipython = QtWidgets.QToolButton()
-        self._button_ipython.setCheckable(True)
-        self._button_ipython.setText("IPython Terminal")
-        self._button_ipython.setIcon(get_icon('IPythonConsole'))
-        self._button_ipython.setToolButtonStyle(Qt.ToolButtonTextBesideIcon)
-        self._button_ipython.clicked.connect(self._toggle_terminal)
-
-        self._data_toolbar.addWidget(self._button_ipython)
-
         self._button_open_session = QtWidgets.QToolButton()
         self._button_open_session.setText("Open Session")
         self._button_open_session.setIcon(get_icon('glue_open'))
@@ -387,24 +394,42 @@ class GlueApplication(Application, QtWidgets.QMainWindow):
         self._button_save_session.clicked.connect(self._choose_save_session)
 
         self._data_toolbar.addWidget(self._button_save_session)
+        self._button_open_data = QtWidgets.QToolButton()
+        self._button_open_data.setText("Import Data")
+        self._button_open_data.setIcon(get_icon('glue_open'))
+        self._button_open_data.setToolButtonStyle(Qt.ToolButtonTextBesideIcon)
+        self._button_open_data.clicked.connect(self._import_helper._choose_load_data_wizard)
+
+        self._data_toolbar.addWidget(self._button_open_data)
+
+        self._button_save_data = QtWidgets.QToolButton()
+        self._button_save_data.setText("Export Data/Subsets")
+        self._button_save_data.setIcon(get_icon('glue_filesave'))
+        self._button_save_data.setToolButtonStyle(Qt.ToolButtonTextBesideIcon)
+        self._button_save_data.clicked.connect(self._choose_save_data)
+
+        self._data_toolbar.addWidget(self._button_save_data)
+
+        self._button_link_data = QtWidgets.QToolButton()
+        self._button_link_data.setText("Link Data")
+        self._button_link_data.setIcon(get_icon('glue_link'))
+        self._button_link_data.setToolButtonStyle(Qt.ToolButtonTextBesideIcon)
+        self._button_link_data.clicked.connect(self._set_up_links)
+
+        self._data_toolbar.addWidget(self._button_link_data)
 
         self._button_edit_components = QtWidgets.QToolButton()
-        self._button_edit_components.setText("Add/edit arithmetic attributes")
-        self._button_edit_components.setIcon(get_icon('pencil'))
+        self._button_edit_components.setText("Arithmetic attributes")
+        self._button_edit_components.setIcon(get_icon('arithmetic'))
         self._button_edit_components.setToolButtonStyle(Qt.ToolButtonTextBesideIcon)
-        self._button_edit_components.clicked.connect(nonpartial(self._layer_widget._create_component))
+        self._button_edit_components.clicked.connect(self._artihmetic_dialog)
 
         self._data_toolbar.addWidget(self._button_edit_components)
 
-        spacer = QtWidgets.QWidget()
-        spacer.setMinimumSize(20, 10)
-        spacer.setSizePolicy(QtWidgets.QSizePolicy.Fixed,
-                             QtWidgets.QSizePolicy.Preferred)
-
-        self._data_toolbar.addWidget(spacer)
-
         self.addToolBar(self._data_toolbar)
 
+        self._on_data_collection_change()
+
         # Selection mode toolbar
 
         tbar = EditSubsetModeToolBar(parent=self)
@@ -414,9 +439,9 @@ class GlueApplication(Application, QtWidgets.QMainWindow):
 
         # Error console toolbar
 
-        self._console_toolbar = QtWidgets.QToolBar()
+        self._console_toolbar = QtWidgets.QToolBar('Advanced toolbar')
 
-        self._console_toolbar.setIconSize(QtCore.QSize(16, 16))
+        self._console_toolbar.setIconSize(QtCore.QSize(14, 14))
 
         spacer = QtWidgets.QWidget()
         spacer.setSizePolicy(QtWidgets.QSizePolicy.Expanding,
@@ -424,6 +449,15 @@ class GlueApplication(Application, QtWidgets.QMainWindow):
 
         self._console_toolbar.addWidget(spacer)
 
+        self._button_ipython = QtWidgets.QToolButton()
+        self._button_ipython.setCheckable(True)
+        self._button_ipython.setText("Terminal")
+        self._button_ipython.setIcon(get_icon('IPythonConsole'))
+        self._button_ipython.setToolButtonStyle(Qt.ToolButtonTextBesideIcon)
+        self._button_ipython.clicked.connect(self._toggle_terminal)
+
+        self._console_toolbar.addWidget(self._button_ipython)
+
         self._button_preferences = QtWidgets.QToolButton()
         self._button_preferences.setText("Preferences")
         self._button_preferences.setIcon(get_icon('glue_settings'))
@@ -433,7 +467,7 @@ class GlueApplication(Application, QtWidgets.QMainWindow):
         self._console_toolbar.addWidget(self._button_preferences)
 
         self._button_console = QtWidgets.QToolButton()
-        self._button_console.setText("View Error Console")
+        self._button_console.setText("Error Console")
         self._button_console.setToolButtonStyle(Qt.ToolButtonTextOnly)
 
         self._console_toolbar.addWidget(self._button_console)
@@ -446,9 +480,19 @@ class GlueApplication(Application, QtWidgets.QMainWindow):
         self._log.hide()
 
         self._hub.subscribe(self, DataCollectionMessage, handler=self._on_data_collection_change)
+        self._hub.subscribe(self, SettingsChangeMessage, handler=self._on_ui_settings_change)
+
+    def _artihmetic_dialog(self, *event):
+        dialog = ArithmeticEditorWidget(self.data_collection)
+        dialog.exec_()
 
     def _on_data_collection_change(self, *event):
+        self._button_save_data.setEnabled(len(self.data_collection) > 0)
         self._button_link_data.setEnabled(len(self.data_collection) > 1)
+        self._button_edit_components.setEnabled(len(self.data_collection) > 0)
+
+    def _on_ui_settings_change(self, *event):
+        update_global_font_size()
 
     def keyPressEvent(self, event):
         if self.current_tab.activeSubWindow() and self.current_tab.activeSubWindow().widget():
@@ -537,22 +581,24 @@ class GlueApplication(Application, QtWidgets.QMainWindow):
         if self.tab_widget.count() == 1:
             return
 
-        if warn and not os.environ.get('GLUE_TESTING'):
-            buttons = QtWidgets.QMessageBox.Ok | QtWidgets.QMessageBox.Cancel
-            dialog = QtWidgets.QMessageBox.warning(
-                self, "Confirm Close",
-                "Are you sure you want to close this tab? "
-                "This will close all data viewers in the tab.",
-                buttons=buttons, defaultButton=QtWidgets.QMessageBox.Cancel)
-            if not dialog == QtWidgets.QMessageBox.Ok:
-                return
-
         w = self.tab_widget.widget(index)
 
-        for window in w.subWindowList():
-            widget = window.widget()
-            if isinstance(widget, DataViewer):
-                widget.close(warn=False)
+        if len(w.subWindowList()) > 0:
+
+            if warn and not os.environ.get('GLUE_TESTING'):
+                buttons = QtWidgets.QMessageBox.Ok | QtWidgets.QMessageBox.Cancel
+                dialog = QtWidgets.QMessageBox.warning(
+                    self, "Confirm Close",
+                    "Are you sure you want to close this tab? "
+                    "This will close all data viewers in the tab.",
+                    buttons=buttons, defaultButton=QtWidgets.QMessageBox.Cancel)
+                if not dialog == QtWidgets.QMessageBox.Ok:
+                    return
+
+            for window in w.subWindowList():
+                widget = window.widget()
+                if isinstance(widget, DataViewer):
+                    widget.close(warn=False)
 
         w.close()
 
@@ -774,23 +820,6 @@ class GlueApplication(Application, QtWidgets.QMainWindow):
         window.show()
         window.exec_()
 
-    def _choose_load_data_wizard(self, *args):
-        self._choose_load_data(data_importer=data_wizard)
-
-    def _choose_load_data(self, data_importer=None):
-        if data_importer is None:
-            self.add_datasets(self.data_collection, data_wizard())
-        else:
-            data = data_importer()
-            if not isinstance(data, list):
-                raise TypeError("Data loader should return list of "
-                                "Data objects")
-            for item in data:
-                if not isinstance(item, Data):
-                    raise TypeError("Data loader should return list of "
-                                    "Data objects")
-            self.add_datasets(self.data_collection, data)
-
     def _choose_save_data(self, *args):
         dialog = SaveDataDialog(data_collection=self.data_collection, parent=self)
         dialog.exec_()
@@ -843,7 +872,7 @@ class GlueApplication(Application, QtWidgets.QMainWindow):
         # also add it again below in the Import menu for consistency.
         a = action("&Open Data Set", self, tip="Open a new data set",
                    shortcut=QtGui.QKeySequence.Open)
-        a.triggered.connect(self._choose_load_data_wizard)
+        a.triggered.connect(self._import_helper._choose_load_data_wizard)
         self._actions['data_new'] = a
 
         # We now populate the "Import data" menu
@@ -853,13 +882,12 @@ class GlueApplication(Application, QtWidgets.QMainWindow):
 
         # Add default file loader (later we can add this to the registry)
         a = action("Import from file", self, tip="Import from file")
-        a.triggered.connect(self._choose_load_data_wizard)
+        a.triggered.connect(self._import_helper._choose_load_data_wizard)
         acts.append(a)
 
-        for i in importer:
-            label, data_importer = i
+        for label, data_importer in importer:
             a = action(label, self, tip=label)
-            a.triggered.connect(self._choose_load_data_wizard)
+            a.triggered.connect(nonpartial(self._import_helper._choose_load_data, data_importer))
             acts.append(a)
 
         self._actions['data_importers'] = acts
@@ -979,7 +1007,12 @@ class GlueApplication(Application, QtWidgets.QMainWindow):
         cmd = command.NewDataViewer(viewer=client, data=data)
         return self.do(cmd)
 
-    new_data_viewer = defer_draw(Application.new_data_viewer)
+    @defer_draw
+    def new_data_viewer(self, viewer_class, data=None, state=None):
+        viewer = super(GlueApplication, self).new_data_viewer(viewer_class, data=data, state=state)
+        if viewer is not None:
+            viewer.show()
+        return viewer
 
     def _choose_save_session(self, *args):
         """ Save the data collection and hub to file.
@@ -990,10 +1023,11 @@ class GlueApplication(Application, QtWidgets.QMainWindow):
         # include file filter twice, so it shows up in Dialog
         outfile, file_filter = compat.getsavefilename(
             parent=self, basedir=getattr(self, '_last_session_name', 'session.glu'),
-            filters=("Glue Session with absolute paths to data(*.glu);; "
-                     "Glue Session with relative paths to data(*.glu);; "
+            filters=("Glue Session with absolute paths to data (*.glu);; "
+                     "Glue Session with relative paths to data (*.glu);; "
                      "Glue Session including data (*.glu)"),
-            selectedfilter=getattr(self, '_last_session_filter', ''))
+            selectedfilter=getattr(self, '_last_session_filter',
+                                   'Glue Session with relative paths to data (*.glu)'))
 
         # This indicates that the user cancelled
         if not outfile:
@@ -1306,7 +1340,7 @@ class GlueApplication(Application, QtWidgets.QMainWindow):
             self.tab_bar.setTabText(index, value)
 
     @staticmethod
-    def _choose_merge(data, others):
+    def _choose_merge(data, others, suggested_label):
 
         w = load_ui('merge.ui', None, directory=os.path.dirname(__file__))
         w.button_yes.clicked.connect(w.accept)
@@ -1328,8 +1362,7 @@ class GlueApplication(Application, QtWidgets.QMainWindow):
                     others[0], others[i] = others[i], others[0]
                     break
 
-        label = others[0].label
-        w.merged_label.setText(label)
+        w.merged_label.setText(suggested_label)
 
         entries = [QtWidgets.QListWidgetItem(other.label) for other in others]
         for e in entries:
@@ -1349,6 +1382,22 @@ class GlueApplication(Application, QtWidgets.QMainWindow):
 
         return None, None
 
+    def screenshot(self, filename):
+        """
+        Save a screenshot of the current application window to a file.
+        """
+        image = QtGui.QImage(self.size(), QtGui.QImage.Format_RGB32)
+        painter = QtGui.QPainter(image)
+        flags = self.IgnoreMask | self.DrawWindowBackground | self.DrawChildren
+        self.render(painter, QtCore.QPoint(), QtGui.QRegion(), flags)
+        image.save(filename)
+        painter.end()
+
+    def add_datasets(self, *args, **kwargs):
+        result = super(GlueApplication, self).add_datasets(*args, **kwargs)
+        run_autolinker(self.data_collection)
+        return result
+
     def __gluestate__(self, context):
         state = super(GlueApplication, self).__gluestate__(context)
         state['tab_names'] = self.tab_names
