@@ -5,30 +5,23 @@ from __future__ import absolute_import, division, print_function
 import os
 import sys
 
+import pytest
 import numpy as np
 from mock import patch, MagicMock
 
-try:
-    from IPython import __version__ as ipy_version
-except:
-    ipy_version = '0.0'
-
 from qtpy import QtCore
 from glue.core.data import Data
 from glue.core.component_link import ComponentLink
 from glue.core.data_collection import DataCollection
-from glue.core.tests.test_state import Cloner, containers_equal, doubler, clone
-from glue.tests.helpers import requires_ipython
-from glue.utils.qt import process_dialog
-from glue.viewers.image.qt import ImageWidget
-from glue.viewers.scatter.qt import ScatterWidget
-from glue.viewers.histogram.qt import HistogramWidget
+from glue.core.tests.test_state import Cloner, doubler, clone
+from glue.tests.helpers import requires_ipython  # noqa
+from glue.viewers.image.qt import ImageViewer
+from glue.viewers.scatter.qt import ScatterViewer
+from glue.viewers.histogram.qt import HistogramViewer
+from glue.utils.qt import process_events
 
 
-from ..application import GlueApplication
-
-
-os.environ['GLUE_TESTING'] = 'True'
+from ..application import GlueApplication, GlueLogger
 
 
 def tab_count(app):
@@ -43,26 +36,31 @@ class TestGlueApplication(object):
 
     def teardown_method(self, method):
         self.app.close()
+        self.app = None
 
     def test_new_tabs(self):
         t0 = tab_count(self.app)
         self.app.new_tab()
         assert tab_count(self.app) == t0 + 1
 
-    def test_save_session(self):
-        self.app.save_session = MagicMock()
-        with patch('qtpy.compat.getsavefilename') as fd:
-            fd.return_value = '/tmp/junk', 'jnk'
-            self.app._choose_save_session()
-            self.app.save_session.assert_called_once_with('/tmp/junk.glu', include_data=False)
+    def test_save_session_ok(self):
+        with patch.object(self.app, 'save_session') as save:
+            with patch('qtpy.compat.getsavefilename') as fd:
+                fd.return_value = '/tmp/junk', 'jnk'
+                self.app._choose_save_session()
+                save.assert_called_once_with('/tmp/junk.glu', include_data=False, absolute_paths=False)
+                fd.reset_mock()
+            save.reset_mock()
 
     def test_save_session_cancel(self):
         """shouldnt try to save file if no file name provided"""
-        self.app.save_session = MagicMock()
-        with patch('qtpy.compat.getsavefilename') as fd:
-            fd.return_value = '', 'jnk'
-            self.app._choose_save_session()
-            assert self.app.save_session.call_count == 0
+        with patch.object(self.app, 'save_session') as save:
+            with patch('glue.app.qt.application.compat.getsavefilename') as fd:
+                fd.return_value = '', 'jnk'
+                self.app._choose_save_session()
+                assert save.call_count == 0
+                fd.reset_mock()
+            save.reset_mock()
 
     def test_choose_save_session_ioerror(self):
         """should show box on ioerror"""
@@ -74,9 +72,12 @@ class TestGlueApplication(object):
             with patch(mock_open) as op:
                 op.side_effect = IOError
                 fd.return_value = '/tmp/junk', '/tmp/junk'
-                with patch('glue.app.qt.application.QMessageBox') as mb:
+                with patch('qtpy.QtWidgets.QMessageBox') as mb:
                     self.app._choose_save_session()
                     assert mb.call_count == 1
+                    mb.reset_mock()
+                op.reset_mock()
+            fd.reset_mock()
 
     @requires_ipython
     def test_terminal_present(self):
@@ -85,46 +86,26 @@ class TestGlueApplication(object):
             sys.stderr.write(self.app._terminal_exception)
             assert False
 
-    def app_without_terminal(self):
-        if not self.app.has_terminal():
-            return self.app
-
-        with patch('glue.app.qt.terminal.glue_terminal') as terminal:
-            terminal.side_effect = Exception("disabled")
-            app = GlueApplication()
-            app._create_terminal()
-            return app
-
-    def test_functional_without_terminal(self):
-        """Can still create app without terminal"""
-        app = self.app_without_terminal()
-
-    def test_messagebox_on_disabled_terminal(self):
-        """Clicking on the terminal toggle button raises messagebox on error"""
-        app = self.app_without_terminal()
-        with patch('glue.app.qt.application.QMessageBox') as qmb:
-            app._terminal_button.click()
-            assert qmb.call_count == 1
-
     def is_terminal_importable(self):
-        try:
-            import glue.qt.widgets.glue_terminal
-            return True
-        except:
-            return False
+        import glue.qt.widgets.glue_terminal  # noqa
 
     @requires_ipython
     def test_toggle_terminal(self):
-        term = MagicMock()
-        self.app._terminal = term
+        with patch.object(self.app, '_terminal') as term:
 
-        term.isVisible.return_value = False
-        self.app._terminal_button.click()
-        assert term.show.call_count == 1
+            self.app._terminal = term
 
-        term.isVisible.return_value = True
-        self.app._terminal_button.click()
-        assert term.hide.call_count == 1
+            term.isVisible.return_value = False
+            with pytest.warns(UserWarning, match='An unexpected error'):
+                self.app._button_ipython.click()
+            assert term.show.call_count == 1
+
+            term.isVisible.return_value = True
+            with pytest.warns(UserWarning, match='An unexpected error'):
+                self.app._button_ipython.click()
+            assert term.hide.call_count == 1
+
+            term.reset_mock()
 
     def test_close_tab(self):
 
@@ -158,28 +139,37 @@ class TestGlueApplication(object):
             self.app.choose_new_data_viewer()
             assert len(self.app.current_tab.subWindowList()) == ct
 
-    def test_new_data_viewer(self):
+            pc.reset_mock()
+
+    def test_new_data_viewer_ok(self):
+
         with patch('glue.app.qt.application.pick_class') as pc:
 
-            pc.return_value = ScatterWidget
+            pc.return_value = ScatterViewer
 
             ct = len(self.app.current_tab.subWindowList())
 
-            self.app.choose_new_data_viewer()
+            viewer = self.app.choose_new_data_viewer()
             assert len(self.app.current_tab.subWindowList()) == ct + 1
+            viewer.close()
+            # TODO: figure out why this doesn't work as expected
+            # assert len(self.app.current_tab.subWindowList()) == ct
+
+            pc.reset_mock()
 
     def test_move(self):
-        viewer = self.app.new_data_viewer(ScatterWidget)
+        viewer = self.app.new_data_viewer(ScatterViewer)
         viewer.move(10, 20)
         assert viewer.position == (10, 20)
+        viewer.close()
 
     def test_resize(self):
-        viewer = self.app.new_data_viewer(ScatterWidget)
+        viewer = self.app.new_data_viewer(ScatterViewer)
         viewer.viewer_size = (100, 200)
         assert viewer.viewer_size == (100, 200)
+        viewer.close()
 
     def test_new_data_defaults(self):
-        from glue.config import qt_client
 
         with patch('glue.app.qt.application.pick_class') as pc:
             pc.return_value = None
@@ -189,49 +179,87 @@ class TestGlueApplication(object):
 
             self.app.choose_new_data_viewer(data=d1)
             args, kwargs = pc.call_args
-            assert kwargs['default'] is ScatterWidget
+            assert kwargs['default'] is ScatterViewer
 
             self.app.choose_new_data_viewer(data=d2)
             args, kwargs = pc.call_args
-            assert kwargs['default'] is ImageWidget
+            assert kwargs['default'] is ImageViewer
+
+            pc.reset_mock()
 
     def test_drop_load_data(self):
+
+        load_data = MagicMock()
+        load_session = MagicMock()
+        self.app.load_data = load_data
+        self.app.restore_session_and_close = load_session
+
+        e = MagicMock()
+
         m = QtCore.QMimeData()
         m.setUrls([QtCore.QUrl('test.fits')])
-        e = MagicMock()
         e.mimeData.return_value = m
-        load = MagicMock()
-        self.app.load_data = load
+
         self.app.dropEvent(e)
-        assert load.call_count == 1
+        assert load_data.called_once_with('test.fits')
+        assert load_session.call_count == 0
+
+        load_data.reset_mock()
+
+        m = QtCore.QMimeData()
+        m.setUrls([QtCore.QUrl('test1.fits'), QtCore.QUrl('test2.fits')])
+        e.mimeData.return_value = m
+        self.app.dropEvent(e)
+        assert load_data.called_once_with(['test1.fits', 'test2.fits'])
+        assert load_session.call_count == 0
+
+        load_data.reset_mock()
+
+        m = QtCore.QMimeData()
+        m.setUrls([QtCore.QUrl('test.glu')])
+        e.mimeData.return_value = m
+        self.app.dropEvent(e)
+        assert load_data.call_count == 0
+        assert load_session.called_once_with(['test.glu'])
+
+        load_data.reset_mock()
+
+        m = QtCore.QMimeData()
+        m.setUrls([QtCore.QUrl('test.glu'), QtCore.QUrl('test.fits')])
+        e.mimeData.return_value = m
+        with pytest.raises(Exception) as exc:
+            self.app.dropEvent(e)
+        assert exc.value.args[0].startswith("When dragging and dropping files")
+
+        load_data.reset_mock()
 
     def test_subset_facet(self):
         # regression test for 335
 
         act = self.app._layer_widget._actions['facet']
         self.app.data_collection.append(Data(x=[1, 2, 3]))
-        with patch('glue.dialogs.subset_facet.qt.SubsetFacet.exec_'):
+        with patch('glue.dialogs.subset_facet.qt.SubsetFacetDialog.exec_'):
             act._do_action()
 
     # FIXME: The following test fails and causes subsequent issues if run with
     #
     # pytest -s -v -x glue
     #
-    # Need to investigate this, but for now, no solution other than skipping 
+    # Need to investigate this, but for now, no solution other than skipping
     # the test.
     #
     # def test_suggest_merge(self):
-    # 
+    #
     #     x = Data(x=[1, 2, 3], label='x')
     #     y = Data(y=[4, 5, 6, 7], label='y')
     #     z = Data(z=[8, 9, 10], label='z')
-    # 
+    #
     #     self.app.data_collection.append(x)
     #     self.app.data_collection.append(y)
-    # 
+    #
     #     with process_dialog(delay=500, accept=True):
     #         result = self.app.add_datasets(self.app.data_collection, z)
-    # 
+    #
     #     np.testing.assert_equal(self.app.data_collection[0]['x'], [1, 2, 3])
     #     np.testing.assert_equal(self.app.data_collection[0]['z'], [8, 9, 10])
     #     np.testing.assert_equal(self.app.data_collection[1]['y'], [4, 5, 6, 7])
@@ -268,13 +296,13 @@ def check_clone_app(app):
             assert v1.position == v2.position
 
             # same viewer-level properties (axis label, scaling, etc)
-            assert set(v1.properties.keys()) == set(v2.properties.keys())
-            for k in v1.properties:
-                if hasattr(v1.properties[k], 'label'):
-                    assert v1.properties[k].label == v2.properties[k].label
-                else:
-                    assert v1.properties[k] == v2.properties[k] or \
-                        containers_equal(v1.properties[k], v2.properties[k])
+            # assert set(v1.properties.keys()) == set(v2.properties.keys())
+            # for k in v1.properties:
+            #     if hasattr(v1.properties[k], 'label'):
+            #         assert v1.properties[k].label == v2.properties[k].label
+            #     else:
+            #         assert v1.properties[k] == v2.properties[k] or \
+            #             containers_equal(v1.properties[k], v2.properties[k])
 
             assert len(v1.layers) == len(v2.layers)
             for l1, l2 in zip(v1.layers, v2.layers):
@@ -293,9 +321,16 @@ class TestApplicationSession(object):
         app = GlueApplication()
         self.check_clone(app)
 
+    def test_tab_names(self):
+        app = GlueApplication()
+        app.tab_bar.setTabText(0, 'Banana')
+        assert app.tab_names == ['Banana']
+        app2 = self.check_clone(app)
+        assert app2.tab_names == ['Banana']
+
     def test_data_application(self):
         dc = DataCollection([Data(label='test',
-                                            x=[1, 2, 3], y=[2, 3, 4])])
+                                  x=[1, 2, 3], y=[2, 3, 4])])
         app = GlueApplication(dc)
         self.check_clone(app)
 
@@ -315,45 +350,57 @@ class TestApplicationSession(object):
         d = Data(label='x', x=[1, 2, 3, 4, 5], y=[2, 3, 4, 5, 6])
         dc = DataCollection([d])
         app = GlueApplication(dc)
-        w = app.new_data_viewer(ScatterWidget, data=d)
-        self.check_clone(app)
+        w = app.new_data_viewer(ScatterViewer, data=d)
+        copy1 = self.check_clone(app)
 
-        s1 = dc.new_subset_group()
-        s2 = dc.new_subset_group()
+        dc.new_subset_group()
+        dc.new_subset_group()
         assert len(w.layers) == 3
         l1, l2, l3 = w.layers
         l1.zorder, l2.zorder = l2.zorder, l1.zorder
         l3.visible = False
         assert l3.visible is False
-        copy = self.check_clone(app)
-        assert copy.viewers[0][0].layers[-1].visible is False
+        copy2 = self.check_clone(app)
+        assert copy2.viewers[0][0].layers[-1].visible is False
+
+        app.close()
+        copy1.close()
+        copy2.close()
 
     def test_multi_tab(self):
         d = Data(label='hist', x=[[1, 2], [2, 3]])
         dc = DataCollection([d])
 
         app = GlueApplication(dc)
-        w1 = app.new_data_viewer(HistogramWidget, data=d)
+        w1 = app.new_data_viewer(HistogramViewer, data=d)
         app.new_tab()
-        w2 = app.new_data_viewer(HistogramWidget, data=d)
+        w2 = app.new_data_viewer(HistogramViewer, data=d)
         assert app.viewers == ((w1,), (w2,))
 
-        self.check_clone(app)
+        copy = self.check_clone(app)
+
+        app.close()
+        copy.close()
 
     def test_histogram(self):
         d = Data(label='hist', x=[[1, 2], [2, 3]])
         dc = DataCollection([d])
 
         app = GlueApplication(dc)
-        w = app.new_data_viewer(HistogramWidget, data=d)
-        self.check_clone(app)
+        w = app.new_data_viewer(HistogramViewer, data=d)
+        copy1 = self.check_clone(app)
 
         dc.new_subset_group()
         assert len(w.layers) == 2
-        self.check_clone(app)
+        copy2 = self.check_clone(app)
 
         w.nbins = 7
-        self.check_clone(app)
+        copy3 = self.check_clone(app)
+
+        app.close()
+        copy1.close()
+        copy2.close()
+        copy3.close()
 
     def test_subset_groups_remain_synced_after_restore(self):
         # regrssion test for 352
@@ -368,3 +415,92 @@ class TestApplicationSession(object):
 
         sg.style.color = '#112233'
         assert sg.subsets[0].style.color == '#112233'
+
+    def test_deselect_tool_on_viewer_change(self):
+
+        d = Data(label='hist', x=[[1, 2], [2, 3]])
+        dc = DataCollection([d])
+
+        app = GlueApplication(dc)
+        v1 = app.new_data_viewer(HistogramViewer, data=d)
+        v2 = app.new_data_viewer(HistogramViewer, data=d)
+
+        assert v1.toolbar.active_tool is None
+        assert v2.toolbar.active_tool is None
+
+        v2.toolbar.active_tool = 'select:xrange'
+
+        assert v1.toolbar.active_tool is None
+        assert v2.toolbar.active_tool.tool_id == 'select:xrange'
+
+        app.current_tab.activateNextSubWindow()
+
+        assert v1.toolbar.active_tool is None
+        assert v2.toolbar.active_tool is None
+
+        v1.toolbar.active_tool = 'select:xrange'
+
+        # Emit a signal without changing the active subWindow to make sure that
+        # the tool doesn't get reset.
+        app.current_tab.subWindowActivated.emit(app.current_tab.activeSubWindow())
+
+        assert v1.toolbar.active_tool.tool_id == 'select:xrange'
+        assert v2.toolbar.active_tool is None
+
+        app.current_tab.activateNextSubWindow()
+
+        assert v1.toolbar.active_tool is None
+        assert v2.toolbar.active_tool is None
+
+        app.close()
+
+    def test_screenshot(self, tmpdir):
+        filename = tmpdir.join('screenshot.png').strpath
+        app = GlueApplication()
+        app.screenshot(filename)
+        assert os.path.exists(filename)
+
+
+def test_logger_close():
+
+    # Regression test to make sure that when closing an application, sys.stderr
+    # no longer points to GlueLogger.
+
+    app = GlueApplication()
+    app.close()
+
+    process_events()
+
+    assert not isinstance(sys.stderr, GlueLogger)
+
+
+def test_reset_session_terminal():
+
+    # Regression test to make sure that the terminal still works when
+    # resetting a session
+
+    app = GlueApplication()
+    app2 = app._reset_session(warn=False)
+
+    assert app2.has_terminal()
+
+    app.close()
+    app2.close()
+
+
+def test_open_session_terminal(tmpdir):
+
+    # Regression test to make sure that the terminal still works when
+    # opening a previous session
+
+    session_file = tmpdir.join('test.glu').strpath
+
+    app = GlueApplication()
+    app.save_session(session_file)
+
+    app2 = app.restore_session(session_file)
+
+    assert app2.has_terminal()
+
+    app.close()
+    app2.close()
