@@ -1,12 +1,11 @@
 from __future__ import absolute_import, division, print_function
 
-from .client import Client  # noqa
 from .command import Command, CommandStack  # noqa
 from .component import Component  # noqa
 from .component_id import ComponentID  # noqa
 from .component_link import ComponentLink  # noqa
 from .coordinates import Coordinates  # noqa  # noqa
-from .data import Data  # noqa
+from .data import BaseData, BaseCartesianData, Data  # noqa
 from .data_collection import DataCollection  # noqa
 from .hub import Hub, HubListener  # noqa
 from .link_manager import LinkManager  # noqa
