@@ -10,7 +10,7 @@ from glue.utils import (shape_to_string, coerce_numeric,
 
 
 __all__ = ['Component', 'DerivedComponent', 'CategoricalComponent',
-           'CoordinateComponent']
+           'CoordinateComponent', 'DateTimeComponent']
 
 
 class Component(object):
@@ -43,6 +43,8 @@ class Component(object):
         # The actual data
         # subclasses may pass non-arrays here as placeholders.
         if isinstance(data, np.ndarray):
+            if data.dtype.kind == 'M':
+                raise TypeError('DateTimeComponent should be used instead of Component for np.datetime64 arrays')
             data = coerce_numeric(data)
             data.setflags(write=False)  # data is read-only
 
@@ -233,7 +235,8 @@ class CoordinateComponent(Component):
             # convert these straight to world coordinates since the indices
             # of the pixel coordinates are the pixel coordinates themselves.
             if isinstance(view, (tuple, list)) and isinstance(view[0], np.ndarray):
-                return np.array(self._data.coords.pixel2world(*view[::-1])[::-1])[self.axis]
+                axis = self._data.ndim - 1 - self.axis
+                return self._data.coords.pixel2world_single_axis(*view[::-1], axis=axis)
 
             # For 1D arrays, slice can be given as a single slice but we need
             # to wrap it in a list to make the following code work correctly,
