@@ -5,6 +5,7 @@ from collections import OrderedDict
 import abc
 import uuid
 import warnings
+from contextlib import contextmanager
 
 import numpy as np
 import pandas as pd
@@ -391,6 +392,34 @@ class BaseCartesianData(BaseData):
         """
         raise NotImplementedError()
 
+    def compute_fixed_resolution_buffer(self, bounds, target_data=None, target_cid=None,
+                                        subset_state=None, broadcast=True):
+        """
+        Get a fixed-resolution buffer.
+
+        Parameters
+        ----------
+        bounds : list
+            The list of bounds for the fixed resolution buffer. This list should
+            have as many items as there are dimensions in ``target_data``. Each
+            item should either be a scalar value, or a tuple of ``(min, max, nsteps)``.
+        target_data : `~glue.core.Data`, optional
+            The data in whose frame of reference the bounds are defined. Defaults
+            to ``data``.
+        target_cid : `~glue.core.component_id.ComponentID`, optional
+            If specified, gives the component ID giving the component to use for the
+            data values. Alternatively, use ``subset_state`` to get a subset mask.
+        subset_state : `~glue.core.subset.SubsetState`, optional
+            If specified, gives the subset state for which to compute a mask.
+            Alternatively, use ``target_cid`` if you want to get data values.
+        broadcast : bool, optional
+            If `True`, then if a dimension in ``target_data`` for which ``bounds``
+            is not a scalar does not affect any of the dimensions in ``data``,
+            then the final array will be effectively broadcast along this
+            dimension, otherwise an error will be raised.
+        """
+        raise NotImplementedError()
+
     def __getitem__(self, key):
         """
         Shortcut syntax to access the numerical data in a component.
@@ -945,16 +974,25 @@ class Data(BaseCartesianData):
             self._pixel_component_ids.append(cid)
 
     def _update_world_components(self, ndim):
-        for cid in self._world_component_ids[:]:
-            self.remove_component(cid)
-            self._world_component_ids.remove(cid)
-        if self.coords:
-            for i in range(ndim):
-                comp = CoordinateComponent(self, i, world=True)
-                label = self.coords.axis_label(i)
-                cid = self.add_component(comp, label)
-                self._world_component_ids.append(cid)
-            self._set_up_coordinate_component_links(ndim)
+
+        if self.hub:
+            delay_callbacks = self.hub.delay_callbacks
+        else:
+            @contextmanager
+            def delay_callbacks():
+                yield
+
+        with delay_callbacks():
+            for cid in self._world_component_ids[:]:
+                self.remove_component(cid)
+                self._world_component_ids.remove(cid)
+            if self.coords:
+                for i in range(ndim):
+                    comp = CoordinateComponent(self, i, world=True)
+                    label = self.coords.axis_label(i)
+                    cid = self.add_component(comp, label)
+                    self._world_component_ids.append(cid)
+                self._set_up_coordinate_component_links(ndim)
 
     def _set_up_coordinate_component_links(self, ndim):
 
@@ -1591,11 +1629,15 @@ class Data(BaseCartesianData):
             return np.zeros(bins)
 
         if log is not None and log[0]:
+            if xmin < 0 or xmax < 0:
+                return np.zeros(bins)
             xmin = np.log10(xmin)
             xmax = np.log10(xmax)
             x = np.log10(x)
 
         if ndim > 1 and log is not None and log[1]:
+            if ymin < 0 or ymax < 0:
+                return np.zeros(bins)
             ymin = np.log10(ymin)
             ymax = np.log10(ymax)
             y = np.log10(y)
@@ -1615,6 +1657,10 @@ class Data(BaseCartesianData):
             range = [(xmin, xmax), (ymin, ymax)]
             return histogram2d(x, y, range=range, bins=bins, weights=w)
 
+    def compute_fixed_resolution_buffer(self, *args, **kwargs):
+        from .fixed_resolution_buffer import compute_fixed_resolution_buffer
+        return compute_fixed_resolution_buffer(self, *args, **kwargs)
+
     # DEPRECATED
 
     @property
