@@ -1,33 +1,60 @@
-"""These classes define the behavior of how new subset states affect
-    the edit_subset of a Data object.
-
-   The EditSubsetMode is universal in Glue -- all datasets and clients
-   share the same mode. This is enforced by making the base
-   EditSubsetMode object a singleton.
 """
-# pylint: disable=I0011, R0903
+These classes define the behavior of how new subset states affect the
+edit_subset of a Data object.
+"""
 
 from __future__ import absolute_import, division, print_function
 
 import logging
 
 from glue.core.contracts import contract
+from glue.core.message import EditSubsetMessage
 from glue.core.data_collection import DataCollection
 from glue.core.data import Data
-from glue.core.decorators import singleton
 from glue.utils import as_list
 
+__all__ = ['EditSubsetMode', 'NewMode', 'ReplaceMode', 'AndMode', 'OrMode',
+           'XorMode', 'AndNotMode']
+
 
-@singleton
 class EditSubsetMode(object):
-
-    """ Implements how new SubsetStates modify the edit_subset state """
+    """
+    Implements how new SubsetStates modify the edit_subset state
+    """
 
     def __init__(self):
-        self.mode = ReplaceMode
         self.data_collection = None
+        self._mode = ReplaceMode
+        self._edit_subset = []
 
-    def _combine_data(self, data, new_state, add_if_empty=False):
+    @property
+    def mode(self):
+        return self._mode
+
+    @mode.setter
+    def mode(self, value):
+        if value is not self._mode:
+            self._mode = value
+            self._broadcast()
+
+    @property
+    def edit_subset(self):
+        return self._edit_subset
+
+    @edit_subset.setter
+    def edit_subset(self, value):
+        if value is self._edit_subset:
+            return
+        elif value is None:
+            value = []
+        self._edit_subset = value
+        self._broadcast()
+
+    def _broadcast(self):
+        if self.data_collection is not None:
+            self.data_collection.hub.broadcast(EditSubsetMessage(self, self.edit_subset, self.mode))
+
+    def _combine_data(self, new_state, override_mode=None):
         """ Dispatches to the combine method of mode attribute.
 
         The behavior is dependent on the mode it dispatches to.
@@ -36,27 +63,22 @@ class EditSubsetMode(object):
 
         :param edit_subset: The current edit_subset
         :param new_state: The new SubsetState
-        :param add_if_empty: If True and a data set has no subsets,
-                             a new one will be added and assigned
-                             using new-state
+        :param override_mode: Mode to use instead of EditSubsetMode.mode
         """
-        empty = data.edit_subset is None or data.edit_subset == []
-        if add_if_empty and empty:
+        mode = override_mode or self.mode
+        if not self._edit_subset or mode is NewMode:
             if self.data_collection is None:
                 raise RuntimeError("Must set data_collection before "
                                    "calling update")
-            data.edit_subset = self.data_collection.new_subset_group()
-        if empty and not add_if_empty:
-            logging.getLogger(__name__).info("Ignoring subset update")
-            return
-        subs = data.edit_subset
+            self.edit_subset = [self.data_collection.new_subset_group()]
+        subs = self._edit_subset
         for s in as_list(subs):
-            self.mode(s, new_state)
+            mode(s, new_state)
 
     @contract(d='inst($DataCollection, $Data)',
               new_state='isinstance(SubsetState)',
               focus_data='inst($Data)|None')
-    def update(self, d, new_state, focus_data=None):
+    def update(self, d, new_state, focus_data=None, override_mode=None):
         """ Apply a new subset state to editable subsets within a
         :class:`~glue.core.data.Data` or
         :class:`~glue.core.data_collection.DataCollection` instance
@@ -70,23 +92,25 @@ class EditSubsetMode(object):
         :param focus_data: The main data set in focus by the client,
         if relevant. If a data set is in focus and has no subsets,
         a new one will be created using new_state.
+
+        :param override_mode: Mode to use instead of EditSubsetMode.mode
+        :type override_mode: bool
         """
         logging.getLogger(__name__).debug("Update subset for %s", d)
 
-        if isinstance(d, Data):
-            self._combine_data(d, new_state, add_if_empty=d is focus_data)
-        elif isinstance(d, DataCollection):
-            no_editable = all(data.edit_subset is None or
-                              data.edit_subset == []
-                              for data in d)
-            for data in d:
-                doadd = data is focus_data and no_editable
-                self._combine_data(data, new_state, add_if_empty=doadd)
+        if isinstance(d, (Data, DataCollection)):
+            self._combine_data(new_state, override_mode=override_mode)
         else:
             raise TypeError("input must be a Data or DataCollection: %s" %
                             type(d))
 
 
+def NewMode(edit_subset, new_state):
+    """ Replaces edit_subset.subset_state with new_state """
+    logging.getLogger(__name__).debug("New %s", edit_subset)
+    edit_subset.subset_state = new_state.copy()
+
+
 def ReplaceMode(edit_subset, new_state):
     """ Replaces edit_subset.subset_state with new_state """
     logging.getLogger(__name__).debug("Replace %s", edit_subset)
