@@ -12,7 +12,7 @@ from .. import roi as r
 from ..component import CategoricalComponent
 from ..roi import (RectangularROI, UndefinedROI, CircularROI, PolygonalROI, CategoricalROI,
                    MplCircularROI, MplRectangularROI, MplPolygonalROI, MplPickROI, PointROI,
-                   XRangeROI, MplXRangeROI, YRangeROI, MplYRangeROI, RangeROI)
+                   XRangeROI, MplXRangeROI, YRangeROI, MplYRangeROI, RangeROI, Projected3dROI)
 
 
 FIG = Figure()
@@ -373,6 +373,38 @@ class TestPolygon(object):
         assert type(str(self.roi)) == str
 
 
+class TestProjected3dROI(object):
+    # matrix that converts xyzw to yxzw
+    xyzw2yxzw = np.array([[0, 1, 0, 0], [0, 0, 1, 0], [1, 0, 0, 0], [0, 0, 0, 1]])
+    x = [1, 2, 3]
+    y = [2, 3, 4]
+    z = [5, 6, 7]
+    # repeat the arrays, 'rolled over' by 1
+    x_nd = [[1, 3], [2, 1], [3, 2]]
+    y_nd = [[2, 3], [3, 2], [4, 3]]
+    z_nd = [[5, 7], [6, 5], [7, 6]]
+
+    def test_contains2d(self):
+        roi_2d = PolygonalROI(vx=[0.5, 2.5, 2.5, 0.5], vy=[1, 1, 3.5, 3.5])
+        roi = Projected3dROI(roi_2d=roi_2d, projection_matrix=np.eye(4))
+        assert roi.contains(self.x, self.y).tolist() == [True, True, False]
+
+    def test_contains3d(self):
+        roi_2d = PolygonalROI(vx=[1.5, 3.5, 3.5, 1.5], vy=[4, 4, 6.5, 6.5])
+        roi = Projected3dROI(roi_2d=roi_2d, projection_matrix=self.xyzw2yxzw)
+        assert roi.contains3d(self.x, self.y, self.z).tolist() == [True, True, False]
+        assert roi.contains3d(self.x_nd, self.y_nd, self.z_nd).tolist() == [[True, False], [True, True], [False, True]]
+
+    def test_forward(self):
+        # testing the calls that should be forwarded to roi_2d
+        roi_2d = PolygonalROI(vx=[0.5, 2.5, 2.5, 0.5], vy=[1, 1, 3.5, 3.5])
+        roi = Projected3dROI(roi_2d=roi_2d, projection_matrix=np.eye(4))
+
+        assert roi.contains(self.x, self.y).tolist() == roi_2d.contains(self.x, self.y).tolist()
+        assert roi.to_polygon() == roi_2d.to_polygon()
+        assert roi.defined() == roi_2d.defined()
+
+
 class TestCategorical(object):
 
     def test_empty(self):
@@ -419,7 +451,7 @@ class TestCategorical(object):
 
     def test_empty_categories(self):
         roi = CategoricalROI()
-        contains = roi.contains(np.array(['a','b','c']), None)
+        contains = roi.contains(np.array(['a', 'b', 'c']), None)
         np.testing.assert_array_equal(contains, [0, 0, 0])
 
 
@@ -464,16 +496,21 @@ class TestMpl(object):
         assert not self.roi._roi.defined()
 
     def test_canvas_syncs_properly(self):
-        assert self.axes.figure.canvas.draw.call_count == 1
+        assert self.axes.figure.canvas.draw_idle.call_count == 0
+        assert self.axes.figure.canvas.blit.call_count == 0
         event = DummyEvent(5, 5, inaxes=self.axes)
         self.roi.start_selection(event)
-        assert self.axes.figure.canvas.draw.call_count == 3
+        assert self.axes.figure.canvas.draw_idle.call_count == 0
+        assert self.axes.figure.canvas.blit.call_count == 1
         self.roi.update_selection(event)
-        assert self.axes.figure.canvas.draw.call_count == 4
+        assert self.axes.figure.canvas.draw_idle.call_count == 0
+        assert self.axes.figure.canvas.blit.call_count == 2
         self.roi.update_selection(event)
-        assert self.axes.figure.canvas.draw.call_count == 5
+        assert self.axes.figure.canvas.draw_idle.call_count == 0
+        assert self.axes.figure.canvas.blit.call_count == 3
         self.roi.finalize_selection(event)
-        assert self.axes.figure.canvas.draw.call_count == 6
+        assert self.axes.figure.canvas.draw_idle.call_count == 0
+        assert self.axes.figure.canvas.blit.call_count == 4
 
     def test_patch_shown_on_start(self):
         assert not self.roi._patch.get_visible()
@@ -964,4 +1001,5 @@ class TestUtil(object):
         assert_almost_equal(pixel[0, 0], xp, 3)
         assert_almost_equal(pixel[0, 1], yp, 3)
 
+
 del TestMpl  # prevents unittest discovery from running abstract base class
