@@ -1,9 +1,10 @@
 from __future__ import absolute_import, division, print_function
 
-from qtpy.QtCore import Qt
-from qtpy import QtWidgets, QtGui
-from glue.utils.qt import mpl_to_qt4_color, tint_pixmap
+from qtpy import QtGui
 
+from matplotlib.colors import Colormap
+
+from glue.utils.qt import mpl_to_qt_color, tint_pixmap, cmap2pixmap
 from glue.icons import icon_path
 
 __all__ = ['symbol_icon', 'layer_icon', 'layer_artist_icon', 'get_icon', 'POINT_ICONS']
@@ -35,7 +36,7 @@ def layer_icon(layer):
     """
     icon = POINT_ICONS.get(layer.style.marker, 'circle_point')
     bm = QtGui.QBitmap(icon_path(icon))
-    color = mpl_to_qt4_color(layer.style.color)
+    color = mpl_to_qt_color(layer.style.color)
     pm = tint_pixmap(bm, color)
     return QtGui.QIcon(pm)
 
@@ -45,18 +46,21 @@ def layer_artist_icon(artist):
 
     # TODO: need a test for this
 
-    from glue.viewers.image.layer_artist import ImageLayerArtist
+    from glue.viewers.scatter.layer_artist import ScatterLayerArtist
 
-    if not artist.enabled:
-        bm = QtGui.QBitmap(icon_path('glue_delete'))
-    elif isinstance(artist, ImageLayerArtist):
-        bm = QtGui.QBitmap(icon_path('glue_image'))
+    color = artist.get_layer_color()
+
+    if isinstance(color, Colormap):
+        pm = cmap2pixmap(color)
     else:
-        bm = QtGui.QBitmap(icon_path(POINT_ICONS.get(artist.layer.style.marker,
-                                                     'glue_circle_point')))
-    color = mpl_to_qt4_color(artist.layer.style.color)
+        if isinstance(artist, ScatterLayerArtist):
+            bm = QtGui.QBitmap(icon_path(POINT_ICONS.get(artist.layer.style.marker,
+                                                         'glue_circle_point')))
+        else:
+            bm = QtGui.QBitmap(icon_path('glue_box_point'))
+        color = mpl_to_qt_color(color)
+        pm = tint_pixmap(bm, color)
 
-    pm = tint_pixmap(bm, color)
     return QtGui.QIcon(pm)
 
 
@@ -74,4 +78,4 @@ def get_icon(icon_name):
     -------
     A QtGui.QIcon object
     """
-    return QtGui.QIcon(icon_path(icon_name))
\ No newline at end of file
+    return QtGui.QIcon(icon_path(icon_name))
