@@ -60,6 +60,8 @@ def parse(argv):
                       help="Startup actions to carry out", default='')
     parser.add_option('--auto-merge', dest='auto_merge', action='store_true',
                       help="Automatically merge any data passed on the command-line", default='')
+    parser.add_option('--faulthandler', dest='faulthandler', action='store_true',
+                      help="Run glue with the built-in faulthandler to debug segmentation faults", default=False)
 
     err_msg = verify(parser, argv)
     if err_msg:
@@ -171,8 +173,8 @@ def start_glue(gluefile=None, config=None, datafiles=None, maximized=True,
 
     if gluefile is not None:
         with die_on_error("Error restoring Glue session"):
-            app = GlueApplication.restore_session(gluefile)
-        return app.start()
+            app = GlueApplication.restore_session(gluefile, show=False)
+        return app.start(maximized=maximized)
 
     if config is not None:
         glue.env = glue.config.load_configuration(search_path=[config])
@@ -188,7 +190,7 @@ def start_glue(gluefile=None, config=None, datafiles=None, maximized=True,
     if datafiles:
         with die_on_error("Error reading data file"):
             datasets = load_data_files(datafiles)
-        ga.add_datasets(data_collection, datasets, auto_merge=auto_merge)
+        ga.add_datasets(datasets, auto_merge=auto_merge)
 
     if startup_actions is not None:
         for name in startup_actions:
@@ -224,6 +226,10 @@ def main(argv=sys.argv):
     if opt.verbose:
         logger.setLevel("INFO")
 
+    if opt.faulthandler:
+        import faulthandler
+        faulthandler.enable()
+
     logger.info("Input arguments: %s", sys.argv)
 
     # Global keywords for Glue startup.
