@@ -1,4 +1,4 @@
-import sys
+from __future__ import absolute_import, division, print_function
 
 
 def load_plugin(plugin):
