@@ -19,7 +19,7 @@ class DendrogramOptionsWidget(QtWidgets.QWidget):
         self.ui = load_ui('options_widget.ui', self,
                           directory=os.path.dirname(__file__))
 
-        autoconnect_callbacks_to_qt(viewer_state, self.ui)
+        self._connections = autoconnect_callbacks_to_qt(viewer_state, self.ui)
 
         self.viewer_state = viewer_state
 
