@@ -2,11 +2,10 @@ from __future__ import absolute_import, division, print_function
 
 import numpy as np
 
-from glue.viewers.common.qt.mouse_mode import PathMode
-from glue.viewers.image.qt import StandaloneImageWidget
-from glue.viewers.common.qt.mpl_widget import defer_draw
-from glue.external.echo import add_callback
+from glue.viewers.matplotlib.toolbar_mode import PathMode
+from glue.viewers.image.qt import StandaloneImageViewer
 from glue.config import viewer_tool
+from glue.utils import defer_draw
 
 
 @viewer_tool
@@ -18,19 +17,21 @@ class PVSlicerMode(PathMode):
     tool_tip = ('Extract a slice from an arbitrary path\n'
                 '  ENTER accepts the path\n'
                 '  ESCAPE clears the path')
+    status_tip = 'Draw a path then press ENTER to extract slice, or press ESC to cancel'
     shortcut = 'P'
 
     def __init__(self, viewer, **kwargs):
         super(PVSlicerMode, self).__init__(viewer, **kwargs)
-        add_callback(viewer.client, 'display_data', self._display_data_hook)
         self._roi_callback = self._extract_callback
         self._slice_widget = None
+        self.viewer.state.add_callback('reference_data', self._on_reference_data_change)
 
-    def _display_data_hook(self, data):
-        if data is not None:
-            self.enabled = data.ndim > 2
+    def _on_reference_data_change(self, reference_data):
+        if reference_data is not None:
+            self.enabled = reference_data.ndim == 3
 
     def _clear_path(self):
+        self.viewer.hide_crosshairs()
         self.clear()
 
     def _extract_callback(self, mode):
@@ -41,12 +42,12 @@ class PVSlicerMode(PathMode):
         self._build_from_vertices(vx, vy)
 
     def _build_from_vertices(self, vx, vy):
-        pv_slice, x, y, wcs = _slice_from_path(vx, vy, self.viewer.data,
-                                               self.viewer.attribute,
-                                               self.viewer.slice)
+        pv_slice, x, y, wcs = _slice_from_path(vx, vy, self.viewer.state.reference_data,
+                                               self.viewer.state.layers[0].attribute,
+                                               self.viewer.state.wcsaxes_slice[::-1])
         if self._slice_widget is None:
             self._slice_widget = PVSliceWidget(image=pv_slice, wcs=wcs,
-                                               image_client=self.viewer.client,
+                                               image_viewer=self.viewer,
                                                x=x, y=y, interpolation='nearest')
             self.viewer._session.application.add_widget(self._slice_widget,
                                                         label='Custom Slice')
@@ -56,8 +57,12 @@ class PVSlicerMode(PathMode):
                                          x=x, y=y, interpolation='nearest')
 
         result = self._slice_widget
-        result.axes.set_xlabel("Position Along Slice")
-        result.axes.set_ylabel(_slice_label(self.viewer.data, self.viewer.slice))
+        result.axes.set_xlabel("Position along path")
+        if wcs is None:
+            result.axes.set_ylabel("Cube slice index")
+        else:
+            result.axes.set_ylabel(_slice_label(self.viewer.state.reference_data,
+                                                self.viewer.state.wcsaxes_slice[::-1]))
 
         result.show()
 
@@ -67,20 +72,20 @@ class PVSlicerMode(PathMode):
         return super(PVSlicerMode, self).close()
 
 
-class PVSliceWidget(StandaloneImageWidget):
+class PVSliceWidget(StandaloneImageViewer):
 
     """ A standalone image widget with extra interactivity for PV slices """
 
-    def __init__(self, image=None, wcs=None, image_client=None,
+    def __init__(self, image=None, wcs=None, image_viewer=None,
                  x=None, y=None, **kwargs):
         """
         :param image: 2D Numpy array representing the PV Slice
         :param wcs: WCS for the PV slice
-        :param image_client: Parent ImageClient this was extracted from
+        :param image_viewer: Parent ImageViewer this was extracted from
         :param kwargs: Extra keywords are passed to imshow
         """
         self._crosshairs = None
-        self._parent = image_client
+        self._parent = image_viewer
         super(PVSliceWidget, self).__init__(image=image, wcs=wcs, **kwargs)
         conn = self.axes.figure.canvas.mpl_connect
         self._down_id = conn('button_press_event', self._on_click)
@@ -88,6 +93,8 @@ class PVSliceWidget(StandaloneImageWidget):
         self.axes.format_coord = self._format_coord
         self._x = x
         self._y = y
+        self._parent.state.add_callback('x_att', self.reset)
+        self._parent.state.add_callback('y_att', self.reset)
 
     def _format_coord(self, x, y):
         """
@@ -103,20 +110,21 @@ class PVSliceWidget(StandaloneImageWidget):
         # xyz -> data pixel coords
         # accounts for fact that image might be shown transposed/rotated
         s = list(self._slc)
-        idx = _slice_index(self._parent.display_data, self._slc)
+        idx = _slice_index(self._parent.state.reference_data, self._slc)
         s[s.index('x')] = pix[0]
         s[s.index('y')] = pix[1]
         s[idx] = pix[2]
 
-        labels = self._parent.coordinate_labels(s)
-        return '         '.join(labels)
+        # labels = self._parent.coordinate_labels(s)
+        # return '         '.join(labels)
+        return ''
 
     def set_image(self, image=None, wcs=None, x=None, y=None, **kwargs):
         super(PVSliceWidget, self).set_image(image=image, wcs=wcs, **kwargs)
         self._axes.set_aspect('auto')
         self._axes.set_xlim(-0.5, image.shape[1] - 0.5)
         self._axes.set_ylim(-0.5, image.shape[0] - 0.5)
-        self._slc = self._parent.slice
+        self._slc = self._parent.state.wcsaxes_slice[::-1]
         self._x = x
         self._y = y
 
@@ -125,12 +133,11 @@ class PVSliceWidget(StandaloneImageWidget):
         s = list(self._slc)
         # XXX breaks if display_data changes
         _, _, z = self._pos_in_parent(event)
-        s[_slice_index(self._parent.display_data, s)] = z
-        self._parent.slice = tuple(s)
+        s[_slice_index(self._parent.state.reference_data, s)] = int(z)
+        self._parent.state.slices = tuple(s)
 
     @defer_draw
     def _draw_crosshairs(self, event):
-
         x, y, _ = self._pos_in_parent(event)
         self._parent.show_crosshairs(x, y)
 
@@ -152,7 +159,7 @@ class PVSliceWidget(StandaloneImageWidget):
             ydata = event.ydata
 
         # Find position slice where cursor is
-        ind = np.clip(xdata, 0, self._im_array.shape[1] - 1)
+        ind = int(round(np.clip(xdata, 0, self._im_array.shape[1] - 1)))
 
         # Find pixel coordinate in input image for this slice
         x = self._x[ind]
@@ -170,6 +177,9 @@ class PVSliceWidget(StandaloneImageWidget):
         self._sync_slice(event)
         self._draw_crosshairs(event)
 
+    def reset(self, *args):
+        self.close()
+
 
 def _slice_from_path(x, y, data, attribute, slc):
     """
@@ -208,24 +218,30 @@ def _slice_from_path(x, y, data, attribute, slc):
     _swap(dims, s, ind, 0)
     _swap(dims, s, s.index('y'), 1)
     _swap(dims, s, s.index('x'), 2)
+
     cube = cube.transpose(dims)
 
+    if cube_wcs is not None:
+        cube_wcs = cube_wcs.sub([data.ndim - nx for nx in dims[::-1]])
+
     # slice down from >3D to 3D if needed
-    s = [slice(None)] * 3 + [slc[d] for d in dims[3:]]
+    s = tuple([slice(None)] * 3 + [slc[d] for d in dims[3:]])
     cube = cube[s]
 
     # sample cube
     spacing = 1  # pixel
     x, y = [np.round(_x).astype(int) for _x in p.sample_points(spacing)]
 
+    from astropy.wcs import WCS
+
     try:
         result = extract_pv_slice(cube, path=p, wcs=cube_wcs, order=0)
-    except:  # sometimes pvextractor complains due to wcs. Try to recover
+        wcs = WCS(result.header)
+    except Exception:  # sometimes pvextractor complains due to wcs. Try to recover
         result = extract_pv_slice(cube, path=p, wcs=None, order=0)
+        wcs = None
 
-    from astropy.wcs import WCS
     data = result.data
-    wcs = WCS(result.header)
 
     return data, x, y, wcs
 
@@ -234,9 +250,10 @@ def _slice_index(data, slc):
     """
     The axis over which to extract PV slices
     """
-    return max([i for i in range(len(slc))
-                if isinstance(slc[i], int)],
-               key=lambda x: data.shape[x])
+    for i in range(len(slc)):
+        if np.isreal(slc[i]):
+            return i
+    raise ValueError("Could not find slice index with slc={0}".format(slc))
 
 
 def _slice_label(data, slc):
@@ -249,4 +266,4 @@ def _slice_label(data, slc):
                 was defined
     """
     idx = _slice_index(data, slc)
-    return data.get_world_component_id(idx).label
+    return data.world_component_ids[idx].label
