@@ -2,7 +2,7 @@ from __future__ import absolute_import, division, print_function
 
 from qtpy import compat
 from glue.config import viewer_tool
-from glue.viewers.common.qt.tool import Tool
+from glue.viewers.common.tool import Tool
 
 
 @viewer_tool
