@@ -26,9 +26,7 @@ def make_skipper(module, label=None, version=None):
 ASTROPY_INSTALLED, requires_astropy = make_skipper('astropy',
                                                    label='Astropy')
 
-MATPLOTLIB_GE_14_INSTALLED, requires_matplotlib_ge_14 = make_skipper('matplotlib',
-                                                                     label='Matplotlib >= 1.4',
-                                                                     version='1.4')
+MATPLOTLIB_GE_22, requires_matplotlib_ge_22 = make_skipper('matplotlib', version='2.2')
 
 ASTRODENDRO_INSTALLED, requires_astrodendro = make_skipper('astrodendro')
 
@@ -42,6 +40,8 @@ SKIMAGE_INSTALLED, requires_skimage = make_skipper('skimage',
 
 XLRD_INSTALLED, requires_xlrd = make_skipper('xlrd')
 
+PYAVM_INSTALLED, requires_pyavm = make_skipper('pyavm')
+
 PLOTLY_INSTALLED, requires_plotly = make_skipper('plotly')
 
 IPYTHON_INSTALLED, requires_ipython = make_skipper('IPython')
