@@ -1,9 +1,7 @@
 from __future__ import absolute_import, division, print_function
 
-from subprocess import check_call
 import sys
-
-from mock import patch
+from subprocess import check_call
 
 from glue.tests.helpers import requires_qt
 
@@ -20,40 +18,18 @@ class TestDependency(object):
         d = Dependency('asdfasdf', 'Non-existent module')
         assert not d.installed
 
-    def test_noinstall(self):
-        with patch('glue._deps.check_call') as check_call:
-            d = Dependency('math', 'exists')
-            d.install()
-            assert check_call.call_count == 0
-
-    def test_install(self):
-        with patch('glue._deps.check_call') as check_call:
-            d = Dependency('asdfasdf', 'never exists')
-            d.install()
-            check_call.assert_called_once_with(['pip',
-                                                'install',
-                                                'asdfasdf'])
-
-    def test_install_with_package_arg(self):
-        with patch('glue._deps.check_call') as check_call:
-            d = Dependency('asdfasdf', 'never exists', package='bcd')
-            d.install()
-            check_call.assert_called_once_with(['pip',
-                                                'install',
-                                                'bcd'])
-
     def test_installed_str(self):
         d = Dependency('math', 'info')
         assert str(d) == "                math:\tINSTALLED (unknown version)"
 
     def test_noinstalled_str(self):
-        d = Dependency('asdf', 'info')
-        assert str(d) == "                asdf:\tMISSING (info)"
+        d = Dependency('asdfasdf', 'info')
+        assert str(d) == "            asdfasdf:\tMISSING (info)"
 
     def test_failed_str(self):
-        d = Dependency('asdf', 'info')
+        d = Dependency('asdfasdf', 'info')
         d.failed = True
-        assert str(d) == "                asdf:\tFAILED (info)"
+        assert str(d) == "            asdfasdf:\tFAILED (info)"
 
 
 @requires_qt
@@ -62,15 +38,14 @@ def test_optional_dependency_not_imported():
     Ensure that a GlueApplication instance can be created without
     importing any non-required dependency
     """
-    optional_deps = categories[2:]
+    optional_deps = categories[5:]
     deps = [dep.module for cateogry, deps in optional_deps for dep in deps]
-    deps.extend(['astropy'])
 
     code = """
 class ImportDenier(object):
     __forbidden = set(%s)
 
-    def find_module(self, mod_name, pth):
+    def find_module(self, mod_name, pth=None):
         if pth:
             return
         if mod_name in self.__forbidden:
