@@ -16,7 +16,7 @@ __all__ = ['unique', 'shape_to_string', 'view_shape', 'stack_view',
            'coerce_numeric', 'check_sorted', 'broadcast_to', 'unbroadcast',
            'iterate_chunks', 'combine_slices', 'nanmean', 'nanmedian', 'nansum',
            'nanmin', 'nanmax', 'format_minimal', 'compute_statistic',
-           'categorical_ndarray', 'index_lookup']
+           'categorical_ndarray', 'index_lookup', 'ensure_numerical']
 
 
 def unbroadcast(array):
@@ -54,8 +54,9 @@ def unique(array):
     """
     # numpy.unique doesn't handle mixed-types on python3,
     # so we use pandas
-    U, I = pd.factorize(array, sort=True)
-    return I, U
+    array = np.asarray(array)
+    I, U = pd.factorize(array, sort=True)
+    return U.astype(array.dtype), I
 
 
 def shape_to_string(shape):
@@ -559,6 +560,13 @@ class categorical_ndarray(np.ndarray):
             raise ValueError("method should be None or 'uniform'")
 
 
+def ensure_numerical(values):
+    if isinstance(values, categorical_ndarray):
+        return values.codes
+    else:
+        return values
+
+
 def index_lookup(data, items):
     """
     Lookup which index in items each data value is equal to
