@@ -1,15 +1,17 @@
 from __future__ import absolute_import, division, print_function
 
+from glue.external.six.moves import queue
 import string
 from functools import partial
-from contextlib import contextmanager
 
+from glue.external.echo.callback_container import CallbackContainer
+from glue.external.six import PY2
 from glue.external.six.moves import reduce
 
 
 __all__ = ['DeferredMethod', 'nonpartial', 'lookup_class', 'as_variable_name',
            'as_list', 'file_format', 'CallbackMixin', 'PropertySetMixin',
-           'Pointer', 'defer']
+           'Pointer', 'common_prefix', 'queue_to_list']
 
 
 class DeferredMethod(object):
@@ -51,7 +53,7 @@ def nonpartial(func, *args, **kwargs):
 
     This is used when connecting slots to ``QAction.triggered`` signals, which
     appear to have different signatures, which seem to add and extra argument
-    in PyQt4 but not PySide
+    in PyQt but not PySide
     """
     def result(*a, **k):
         return func(*args, **kwargs)
@@ -61,18 +63,26 @@ def nonpartial(func, *args, **kwargs):
 
 def lookup_class(ref):
     """
-    Look up an object via its module string (e.g., 'glue.core.Data')
+    Look up an object via its module string (e.g., 'glue.core.data.Data')
 
     Parameters
     ----------
     ref : str
         The module string
     """
+
+    if PY2 and ref.startswith('builtins'):
+        ref = '.'.join(['__builtin__'] + ref.split('.')[1:])
+    elif not PY2 and ref.startswith('__builtin__'):
+        ref = '.'.join(['builtins'] + ref.split('.')[1:])
+
     mod = ref.rsplit('.', 1)[0]
+
     try:
         result = __import__(mod)
     except ImportError:
         raise ValueError("Module '{0}' not found".format(mod))
+
     try:
         for attr in ref.split('.')[1:]:
             result = getattr(result, attr)
@@ -126,7 +136,7 @@ class CallbackMixin(object):
     """
 
     def __init__(self):
-        self._callbacks = []
+        self._callbacks = CallbackContainer()
 
     def add_callback(self, function):
         self._callbacks.append(function)
@@ -171,6 +181,22 @@ class PropertySetMixin(object):
             setattr(self, k, value[k])
 
 
+def common_prefix(strings, exclude_punctuation=True):
+    """
+    Given a list of strings, find the longest prefix common to all of them
+    """
+    if len(strings) > 0:
+        for i in range(len(strings[0]), 0, -1):
+            if exclude_punctuation and strings[0][i - 1] in string.punctuation:
+                continue
+            for st in strings[1:]:
+                if st[:i] != strings[0][:i]:
+                    break
+            else:
+                return strings[0][:i]
+    return ''
+
+
 class Pointer(object):
 
     def __init__(self, key):
@@ -188,38 +214,13 @@ class Pointer(object):
         setattr(attr, v[-1], value)
 
 
-# TODO: defer can be removed since it doesn't appear to be used anywhere
-
-@contextmanager
-def defer(instance, method):
+def queue_to_list(q):
     """
-    Defer the calling of a method inside a context manager,
-    and then call it 0 or 1 times afterwards.
-
-    :param instance: The instance of the method to defer
-    :param method: The name of the method to defer
-    :type method: str
-
-    Within the context block, calls to the method will be
-    intercepted, logged, and skipped.
-
-    Upon exiting the context block, the method will be
-    invoked a single time, with the arguments of the
-    most recent invokation inside the context block.
-
-    If the method is never invoked in the context block,
-    it is not called when leaving that block.
+    Get all the values in a :class:`queue.Queue` object and return a list.
     """
-    history = []
-
-    def log(*a, **k):
-        history.append((a, k))
-
-    orig = getattr(instance, method)
-    setattr(instance, method, log)
-    try:
-        yield
-    finally:
-        setattr(instance, method, orig)
-        for a, k in history[-1:]:
-            orig(*a, **k)
+    l = []
+    while True:
+        try:
+            l.append(q.get_nowait())
+        except queue.Empty:
+            return l
