@@ -1,3 +1,3 @@
 from __future__ import absolute_import, division, print_function
 
-from .custom_viewer import *
\ No newline at end of file
+from .custom_viewer import *  # noqa
