@@ -1,7 +1,6 @@
 from __future__ import absolute_import, division, print_function
 
 import sys
-import time
 import warnings
 
 import numpy as np
@@ -15,14 +14,6 @@ from glue.viewers.histogram.python_export import python_export_histogram_layer
 from glue.viewers.matplotlib.layer_artist import MatplotlibLayerArtist
 from glue.core.exceptions import IncompatibleAttribute, IncompatibleDataException
 
-try:
-    import qtpy  # noqa
-except Exception:
-    QT_INSTALLED = False
-else:
-    QT_INSTALLED = True
-    from glue.viewers.matplotlib.qt.compute_worker import ComputeWorker
-
 
 class HistogramLayerArtist(MatplotlibLayerArtist):
 
@@ -39,59 +30,15 @@ class HistogramLayerArtist(MatplotlibLayerArtist):
         self._viewer_state.add_global_callback(self._update_histogram)
         self.state.add_global_callback(self._update_histogram)
 
-        self.reset_cache()
-
-        if QT_INSTALLED:
-            self.setup_thread()
-
-    def wait(self):
-        if QT_INSTALLED:
-            # Wait 0.5 seconds to make sure that the computation has properly started
-            time.sleep(0.5)
-            while self._worker.running:
-                time.sleep(1 / 25)
-            from glue.utils.qt import get_qapp
-            app = get_qapp()
-            app.processEvents()
-
-    def remove(self):
-        super(HistogramLayerArtist, self).remove()
-        if QT_INSTALLED and self._worker is not None:
-            self._worker.work_queue.put('stop')
-            self._worker.exit()
-            # Need to wait otherwise the thread will be destroyed while still
-            # running, causing a segmentation fault
-            self._worker.wait()
-            self._worker = None
-
-    @property
-    def is_computing(self):
-        if QT_INSTALLED:
-            return self._worker.running
-
-    def reset_cache(self):
-        self._last_viewer_state = {}
-        self._last_layer_state = {}
-
-    def setup_thread(self):
-        self._worker = ComputeWorker(self._calculate_histogram_thread)
-        self._worker.compute_end.connect(self._calculate_histogram_postthread)
-        self._worker.compute_error.connect(self._calculate_histogram_error)
-        self._worker.compute_start.connect(self.notify_start_computation)
-        self._worker.start()
-
     @defer_draw
     def _calculate_histogram(self, reset=False):
-        if self.state.layer is not None and self.state.layer.size > 1e7 and QT_INSTALLED:
-            self._worker.work_queue.put(reset)
+        try:
+            self.notify_start_computation()
+            self._calculate_histogram_thread(reset=reset)
+        except Exception:
+            self._calculate_histogram_error(sys.exc_info())
         else:
-            try:
-                self.notify_start_computation()
-                self._calculate_histogram_thread(reset=reset)
-            except Exception:
-                self._calculate_histogram_error(sys.exc_info())
-            else:
-                self._calculate_histogram_postthread()
+            self._calculate_histogram_postthread()
 
     def _calculate_histogram_thread(self, reset=False):
         # We need to ignore any warnings that happen inside the thread
@@ -105,12 +52,6 @@ class HistogramLayerArtist(MatplotlibLayerArtist):
             self.state.update_histogram()
 
     def _calculate_histogram_postthread(self):
-
-        # If the worker has started running again, we should stop at this point
-        # since this function will get called again.
-        if QT_INSTALLED and self._worker.running:
-            return
-
         self.notify_end_computation()
         self._update_artists()
         self._update_visual_attributes()
@@ -222,27 +163,7 @@ class HistogramLayerArtist(MatplotlibLayerArtist):
                 self.state.layer is None):
             return
 
-        # Figure out which attributes are different from before. Ideally we shouldn't
-        # need this but currently this method is called multiple times if an
-        # attribute is changed due to x_att changing then hist_x_min, hist_x_max, etc.
-        # If we can solve this so that _update_histogram is really only called once
-        # then we could consider simplifying this. Until then, we manually keep track
-        # of which properties have changed.
-
-        changed = set()
-
-        if not force:
-
-            for key, value in self._viewer_state.as_dict().items():
-                if value != self._last_viewer_state.get(key, None):
-                    changed.add(key)
-
-            for key, value in self.state.as_dict().items():
-                if value != self._last_layer_state.get(key, None):
-                    changed.add(key)
-
-        self._last_viewer_state.update(self._viewer_state.as_dict())
-        self._last_layer_state.update(self.state.as_dict())
+        changed = set() if force else self.pop_changed_properties()
 
         if force or any(prop in changed for prop in ('layer', 'x_att', 'hist_x_min', 'hist_x_max', 'hist_n_bin', 'x_log', 'y_log', 'normalize', 'cumulative')):
             self._calculate_histogram(reset=force)
