@@ -17,10 +17,11 @@ from glue.core.subset import RangeSubsetState, CategoricalROISubsetState
 from glue import core
 from glue.app.qt import GlueApplication
 from glue.core.component_id import ComponentID
-from glue.utils.qt import combo_as_string
+from glue.utils.qt import combo_as_string, process_events
 from glue.viewers.matplotlib.qt.tests.test_data_viewer import BaseTestMatplotlibDataViewer
 from glue.core.state import GlueUnSerializer
 from glue.app.qt.layer_tree_widget import LayerTreeWidget
+from glue.tests.helpers import requires_matplotlib_ge_22
 
 from ..data_viewer import HistogramViewer
 
@@ -112,6 +113,8 @@ class TestHistogramViewer(object):
         self.viewer.add_data(data)
         viewer_state.x_log = True
 
+        process_events()
+
         labels = [x.get_text() for x in self.viewer.axes.xaxis.get_ticklabels()]
 
         # Different Matplotlib versions return slightly different
@@ -541,6 +544,8 @@ class TestHistogramViewer(object):
         assert viewer4.state.cumulative
         assert not viewer4.state.normalize
 
+        ga.close()
+
     def test_apply_roi_single(self):
 
         # Regression test for a bug that caused mode.update to be called
@@ -587,6 +592,7 @@ class TestHistogramViewer(object):
         for subset in client.count:
             assert client.count[subset] == 1
 
+    @pytest.mark.filterwarnings('ignore:elementwise')
     def test_datetime64_support(self, tmpdir):
 
         self.data.add_component(np.array([100, 200, 300, 400], dtype='M8[D]'), 't1')
@@ -631,3 +637,33 @@ class TestHistogramViewer(object):
 
         assert options.valuetext_x_min.text() == '1970-04-14'
         assert options.valuetext_x_max.text() == '1971-02-05'
+
+        ga.close()
+
+    @requires_matplotlib_ge_22
+    def test_categorical_labels(self, tmpdir):
+
+        # Fix a bug that caused labels on histograms of categorical variables
+        # to not be restored correctly after saving and reloading session
+
+        self.viewer.add_data(self.data)
+        self.viewer.state.x_att = self.data.id['y']
+
+        self.viewer.figure.canvas.draw()
+
+        assert [x.get_text() for x in self.viewer.axes.xaxis.get_ticklabels()] == ['', 'a', 'b', 'c', '']
+
+        # Make sure that everything works fine after saving/reloading
+        filename = tmpdir.join('test_categorical_labels.glu').strpath
+        self.session.application.save_session(filename)
+        with open(filename, 'r') as f:
+            session = f.read()
+        state = GlueUnSerializer.loads(session)
+        ga = state.object('__main__')
+        viewer = ga.viewers[0][0]
+
+        viewer.figure.canvas.draw()
+
+        assert [x.get_text() for x in viewer.axes.xaxis.get_ticklabels()] == ['', 'a', 'b', 'c', '']
+
+        ga.close()
