@@ -13,13 +13,13 @@ from astropy.wcs import WCS
 import numpy as np
 from numpy.testing import assert_allclose
 
-from glue.external.modest_image import ModestImage
+from glue.viewers.image.frb_artist import FRBArtist
 from glue.core.coordinates import Coordinates, WCSCoordinates
 from glue.core.message import SubsetUpdateMessage
 from glue.core import HubListener, Data
 from glue.core.roi import XRangeROI, RectangularROI
 from glue.core.subset import RoiSubsetState
-from glue.utils.qt import combo_as_string
+from glue.utils.qt import combo_as_string, process_events
 from glue.viewers.matplotlib.qt.tests.test_data_viewer import BaseTestMatplotlibDataViewer
 from glue.core.state import GlueUnSerializer
 from glue.app.qt.layer_tree_widget import LayerTreeWidget
@@ -27,6 +27,7 @@ from glue.viewers.scatter.state import ScatterLayerState
 from glue.viewers.image.state import ImageLayerState, ImageSubsetLayerState, AggregateSlice
 from glue.core.link_helpers import LinkSame
 from glue.app.qt import GlueApplication
+from glue.core.fixed_resolution_buffer import ARRAY_CACHE, PIXEL_CACHE
 
 from ..data_viewer import ImageViewer
 
@@ -106,11 +107,19 @@ class TestImageViewer(object):
         self.options_widget = self.viewer.options_widget()
 
     def teardown_method(self, method):
+
+        # Properly close viewer and application
         self.viewer.close()
         self.viewer = None
         self.application.close()
         self.application = None
 
+        # Make sure cache is empty
+        if len(PIXEL_CACHE) > 0:
+            raise Exception("Pixel cache contains {0} elements".format(len(PIXEL_CACHE)))
+        if len(ARRAY_CACHE) > 0:
+            raise Exception("Array cache contains {0} elements".format(len(ARRAY_CACHE)))
+
     def test_basic(self):
 
         # Check defaults when we add data
@@ -315,6 +324,7 @@ class TestImageViewer(object):
         self.viewer.layers[1].visible = False
         self.viewer.layers[1].image_artist.invalidate_cache()
         self.viewer.layers[1].redraw()
+        process_events()
         assert not np.any(self.viewer.layers[1].image_artist._A.mask)
         self.viewer.layers[1].visible = True
         assert not np.any(self.viewer.layers[1].image_artist._A.mask)
@@ -378,6 +388,8 @@ class TestImageViewer(object):
 
         self.data_collection.new_subset_group()
 
+        process_events()
+
         assert len(self.viewer.layers) == 4
 
         # Only the two layers associated with the reference data should be enabled
@@ -393,6 +405,8 @@ class TestImageViewer(object):
         link1 = LinkSame(px1, px2)
         self.data_collection.add_link(link1)
 
+        process_events()
+
         # One link isn't enough, second dataset layers are still not enabled
 
         for layer_artist in self.viewer.layers:
@@ -404,6 +418,8 @@ class TestImageViewer(object):
         link2 = LinkSame(py1, py2)
         self.data_collection.add_link(link2)
 
+        process_events()
+
         # All layers should now be enabled
 
         for layer_artist in self.viewer.layers:
@@ -411,6 +427,8 @@ class TestImageViewer(object):
 
         self.data_collection.remove_link(link2)
 
+        process_events()
+
         # We should now be back to the original situation
 
         for layer_artist in self.viewer.layers:
@@ -513,23 +531,23 @@ class TestImageViewer(object):
         # Regression test for a bug in v0.11.0 that meant that if a subset
         # was removed, the image viewer would then crash when changing view
         # (e.g. zooming in). The bug was caused by undeleted references to
-        # ModestImage due to circular references. We therefore check in this
-        # test how many ModestImage objects exist.
+        # FRBArtist due to circular references. We therefore check in this
+        # test how many FRBArtist objects exist.
 
-        def get_modest_images():
+        def get_frb_artists():
             mi = []
             gc.collect()
             for obj in gc.get_objects():
                 try:
-                    if isinstance(obj, ModestImage):
+                    if isinstance(obj, FRBArtist):
                         mi.append(obj)
                 except ReferenceError:
                     pass
             return mi
 
-        # The viewer starts off with one ModestImage. This is also a good test
-        # that other ModestImages in other tests have been removed.
-        assert len(get_modest_images()) == 1
+        # The viewer starts off with one FRBArtist. This is also a good test
+        # that other FRBArtist in other tests have been removed.
+        assert len(get_frb_artists()) == 1
 
         large_image = Data(x=np.random.random((2048, 2048)))
         self.data_collection.append(large_image)
@@ -540,15 +558,15 @@ class TestImageViewer(object):
         self.viewer.add_data(large_image)
 
         # Since the dataset added has a subset, and each subset has its own
-        # ModestImage, this increases the count.
-        assert len(get_modest_images()) == 2
+        # FRBArtist, this increases the count.
+        assert len(get_frb_artists()) == 2
 
         assert len(self.viewer.layers) == 2
 
         self.data_collection.remove_subset_group(subset_group)
 
         # Removing the subset should bring the count back to 1 again
-        assert len(get_modest_images()) == 1
+        assert len(get_frb_artists()) == 1
 
     def test_select_previously_incompatible_layer(self):
 
@@ -566,6 +584,8 @@ class TestImageViewer(object):
 
         self.data_collection.new_subset_group(subset_state=self.catalog.id['e'] > 4)
 
+        process_events()
+
         assert self.viewer.layers[0].enabled  # image
         assert self.viewer.layers[1].enabled  # scatter
         assert not self.viewer.layers[2].enabled  # image subset
@@ -575,6 +595,8 @@ class TestImageViewer(object):
 
         self.data_collection.subset_groups[0].subset_state = self.catalog.id['c'] > -1
 
+        process_events()
+
         assert self.viewer.layers[0].enabled  # image
         assert self.viewer.layers[1].enabled  # scatter
         assert self.viewer.layers[2].enabled  # image subset
@@ -593,6 +615,8 @@ class TestImageViewer(object):
 
         self.data_collection.new_subset_group(subset_state=self.catalog.id['e'] > 4)
 
+        process_events()
+
         assert self.viewer.layers[0].enabled  # image
         assert not self.viewer.layers[1].enabled  # scatter
         assert not self.viewer.layers[2].enabled  # image subset
@@ -603,6 +627,8 @@ class TestImageViewer(object):
         self.data_collection.add_link(link1)
         self.data_collection.add_link(link2)
 
+        process_events()
+
         assert self.viewer.layers[0].enabled  # image
         assert self.viewer.layers[1].enabled  # scatter
         assert not self.viewer.layers[2].enabled  # image subset
