@@ -16,17 +16,21 @@ from glue.core import Data
 from glue.core.exceptions import IncompatibleDataException
 from glue.app.qt.application import GlueApplication
 from glue.core.roi import XRangeROI
-from glue.utils.qt import get_qapp
+from glue.utils.qt import process_events
+from glue.tests.helpers import requires_matplotlib_ge_22
 
 
 class MatplotlibDrawCounter(object):
 
     def __init__(self, figure):
-        self.draw_count = 0
-        figure.canvas.mpl_connect('draw_event', self.on_draw)
+        self.figure = figure
+        process_events()
+        self.start = self.figure.canvas._draw_count
 
-    def on_draw(self, event):
-        self.draw_count += 1
+    @property
+    def draw_count(self):
+        process_events()
+        return self.figure.canvas._draw_count - self.start
 
 
 class BaseTestMatplotlibDataViewer(object):
@@ -71,8 +75,7 @@ class BaseTestMatplotlibDataViewer(object):
 
     @property
     def viewer_count(self):
-        app = get_qapp()
-        app.processEvents()
+        process_events()
         obj = objgraph.by_type(self.viewer_cls.__name__)
         return len(obj)
 
@@ -532,13 +535,18 @@ class BaseTestMatplotlibDataViewer(object):
         self.data.update_values_from_data(data)
         assert self.draw_count == 2
 
+    @requires_matplotlib_ge_22
     def test_aspect_resize(self):
 
         # Make sure that the limits are adjusted appropriately when resizing
         # depending on the aspect ratio mode. Note that we don't add any data
         # here since it isn't needed for this test.
 
-        app = get_qapp()
+        # This test works with Matplotlib 2.0 and 2.2 but not 2.1, hence we
+        # skip it with Matplotlib 2.1 above.
+
+        # Note that we need to explicitly call draw() below because otherwise
+        # draw_idle is used, which has no guarantee of being effective.
 
         # Set initial limits to deterministic values
         self.viewer.state.aspect = 'auto'
@@ -551,7 +559,8 @@ class BaseTestMatplotlibDataViewer(object):
 
         # Resize events only work if widget is visible
         self.viewer.show()
-        app.processEvents()
+        self.viewer.figure.canvas.draw()
+        process_events(wait=0.1)
 
         def limits(viewer):
             return (viewer.state.x_min, viewer.state.x_max,
@@ -559,28 +568,48 @@ class BaseTestMatplotlibDataViewer(object):
 
         # Set viewer to an initial size and save limits
         self.viewer.viewer_size = (800, 400)
-        app.processEvents()
+        self.viewer.figure.canvas.draw()
+        process_events(wait=0.1)
         initial_limits = limits(self.viewer)
 
         # Change the viewer size, and make sure the limits are adjusted
         self.viewer.viewer_size = (400, 400)
-        app.processEvents()
+        self.viewer.figure.canvas.draw()
+        process_events(wait=0.1)
         with pytest.raises(AssertionError):
             assert_allclose(limits(self.viewer), initial_limits)
 
         # Now change the viewer size a number of times and make sure if we
         # return to the original size, the limits match the initial ones.
         self.viewer.viewer_size = (350, 800)
-        app.processEvents()
+        self.viewer.figure.canvas.draw()
+        process_events(wait=0.1)
         self.viewer.viewer_size = (900, 300)
-        app.processEvents()
+        self.viewer.figure.canvas.draw()
+        process_events(wait=0.1)
         self.viewer.viewer_size = (600, 600)
-        app.processEvents()
+        self.viewer.figure.canvas.draw()
+        process_events(wait=0.1)
         self.viewer.viewer_size = (800, 400)
-        app.processEvents()
+        self.viewer.figure.canvas.draw()
+        process_events(wait=0.1)
         assert_allclose(limits(self.viewer), initial_limits)
 
         # Now check that the limits don't change in 'auto' mode
         self.viewer.state.aspect = 'auto'
         self.viewer.viewer_size = (900, 300)
+        self.viewer.figure.canvas.draw()
+        process_events(wait=0.1)
         assert_allclose(limits(self.viewer), initial_limits)
+
+    def test_update_data_values(self):
+
+        # Regression test for a bug that caused some viewers to not behave
+        # correctly if the data values were updated.
+
+        self.viewer.add_data(self.data)
+
+        data = self.init_data()
+        self.data_collection.append(data)
+
+        self.data.update_values_from_data(data)
