@@ -1,26 +1,16 @@
 from __future__ import absolute_import, division, print_function
 
 import sys
-import time
 import warnings
 
 import numpy as np
 
-from glue.external.six.moves import queue
 from glue.core import BaseData
 from glue.utils import defer_draw, nanmin, nanmax
 from glue.viewers.profile.state import ProfileLayerState
 from glue.viewers.matplotlib.layer_artist import MatplotlibLayerArtist
 from glue.core.exceptions import IncompatibleAttribute, IncompatibleDataException
 
-try:
-    import qtpy  # noqa
-except Exception:
-    QT_INSTALLED = False
-else:
-    QT_INSTALLED = True
-    from glue.viewers.matplotlib.qt.compute_worker import ComputeWorker
-
 
 class ProfileLayerArtist(MatplotlibLayerArtist):
 
@@ -40,60 +30,15 @@ class ProfileLayerArtist(MatplotlibLayerArtist):
 
         self.mpl_artists = [self.plot_artist]
 
-        self.reset_cache()
-
-        if QT_INSTALLED:
-            self.setup_thread()
-
-    def wait(self):
-        if QT_INSTALLED:
-            # Wait 0.5 seconds to make sure that the computation has properly started
-            time.sleep(0.5)
-            while self._worker.running:
-                time.sleep(1 / 25)
-            from glue.utils.qt import get_qapp
-            app = get_qapp()
-            app.processEvents()
-
-    def remove(self):
-        super(ProfileLayerArtist, self).remove()
-        if QT_INSTALLED and self._worker is not None:
-            self._worker.work_queue.put('stop')
-            self._worker.exit()
-            # Need to wait otherwise the thread will be destroyed while still
-            # running, causing a segmentation fault
-            self._worker.wait()
-            self._worker = None
-
-    @property
-    def is_computing(self):
-        if QT_INSTALLED:
-            return self._worker.running
-
-    def reset_cache(self):
-        self._last_viewer_state = {}
-        self._last_layer_state = {}
-
-    def setup_thread(self):
-        self._worker = ComputeWorker(self._calculate_profile_thread)
-        self._worker.compute_end.connect(self._calculate_profile_postthread)
-        self._worker.compute_error.connect(self._calculate_profile_error)
-        self._worker.compute_start.connect(self.notify_start_computation)
-        self._worker.work_queue = queue.Queue()
-        self._worker.start()
-
     @defer_draw
     def _calculate_profile(self, reset=False):
-        if self.state.layer is not None and self.state.layer.size > 1e7 and QT_INSTALLED:
-            self._worker.work_queue.put(reset)
+        try:
+            self.notify_start_computation()
+            self._calculate_profile_thread(reset=reset)
+        except Exception:
+            self._calculate_profile_error(sys.exc_info())
         else:
-            try:
-                self.notify_start_computation()
-                self._calculate_profile_thread(reset=reset)
-            except Exception:
-                self._calculate_profile_error(sys.exc_info())
-            else:
-                self._calculate_profile_postthread()
+            self._calculate_profile_postthread()
 
     def _calculate_profile_thread(self, reset=False):
         # We need to ignore any warnings that happen inside the thread
@@ -108,11 +53,6 @@ class ProfileLayerArtist(MatplotlibLayerArtist):
 
     def _calculate_profile_postthread(self):
 
-        # If the worker has started running again, we should stop at this point
-        # since this function will get called again.
-        if QT_INSTALLED and self._worker.running:
-            return
-
         self.notify_end_computation()
 
         # It's possible for this method to get called but for the state to have
@@ -212,27 +152,7 @@ class ProfileLayerArtist(MatplotlibLayerArtist):
                 self.state.layer is None):
             return
 
-        # Figure out which attributes are different from before. Ideally we shouldn't
-        # need this but currently this method is called multiple times if an
-        # attribute is changed due to x_att changing then hist_x_min, hist_x_max, etc.
-        # If we can solve this so that _update_profile is really only called once
-        # then we could consider simplifying this. Until then, we manually keep track
-        # of which properties have changed.
-
-        changed = set()
-
-        if not force:
-
-            for key, value in self._viewer_state.as_dict().items():
-                if value != self._last_viewer_state.get(key, None):
-                    changed.add(key)
-
-            for key, value in self.state.as_dict().items():
-                if value != self._last_layer_state.get(key, None):
-                    changed.add(key)
-
-        self._last_viewer_state.update(self._viewer_state.as_dict())
-        self._last_layer_state.update(self.state.as_dict())
+        changed = set() if force else self.pop_changed_properties()
 
         if force or any(prop in changed for prop in ('layer', 'x_att', 'attribute', 'function', 'normalize', 'v_min', 'v_max')):
             self._calculate_profile(reset=force)
