@@ -4,11 +4,10 @@ from glue.utils import defer_draw, decorate_all_methods
 
 from glue.viewers.matplotlib.qt.data_viewer import MatplotlibDataViewer
 from glue.viewers.profile.qt.layer_style_editor import ProfileLayerStyleEditor
-from glue.viewers.profile.layer_artist import ProfileLayerArtist
+from glue.viewers.profile.qt.layer_artist import QThreadedProfileLayerArtist
 from glue.viewers.profile.qt.options_widget import ProfileOptionsWidget
 from glue.viewers.profile.state import ProfileViewerState
 
-from glue.viewers.common.qt import toolbar_mode  # noqa
 from glue.viewers.profile.qt.profile_tools import ProfileAnalysisTool  # noqa
 from glue.viewers.profile.viewer import MatplotlibProfileMixin
 
@@ -22,8 +21,8 @@ class ProfileViewer(MatplotlibProfileMixin, MatplotlibDataViewer):
     _layer_style_widget_cls = ProfileLayerStyleEditor
     _state_cls = ProfileViewerState
     _options_cls = ProfileOptionsWidget
-    _data_artist_cls = ProfileLayerArtist
-    _subset_artist_cls = ProfileLayerArtist
+    _data_artist_cls = QThreadedProfileLayerArtist
+    _subset_artist_cls = QThreadedProfileLayerArtist
 
     large_data_size = 1e8
 
