@@ -16,7 +16,7 @@ from glue.utils.qt import load_ui, fix_tab_widget_fontsize
 from glue.viewers.profile.qt.mouse_mode import NavigateMouseMode, RangeMouseMode
 from glue.core.qt.fitters import FitSettingsWidget
 from glue.utils.qt import Worker
-from glue.viewers.common.qt.tool import Tool
+from glue.viewers.common.tool import Tool
 from glue.viewers.image.state import AggregateSlice
 from glue.core.aggregate import mom1, mom2
 from glue.core import BaseData, Subset
@@ -122,11 +122,11 @@ class ProfileTools(QtWidgets.QWidget):
 
         ProfileTools.fit_function.set_choices(self, list(fit_plugin))
         ProfileTools.fit_function.set_display_func(self, lambda fitter: fitter.label)
-        connect_combo_selection(self, 'fit_function', self.ui.combosel_fit_function)
+        self._connection_fit = connect_combo_selection(self, 'fit_function', self.ui.combosel_fit_function)
 
         ProfileTools.collapse_function.set_choices(self, list(COLLAPSE_FUNCS))
         ProfileTools.collapse_function.set_display_func(self, COLLAPSE_FUNCS.get)
-        connect_combo_selection(self, 'collapse_function', self.ui.combosel_collapse_function)
+        self._connection_collapse = connect_combo_selection(self, 'collapse_function', self.ui.combosel_collapse_function)
 
         self._toolbar_connected = False
 
@@ -142,7 +142,7 @@ class ProfileTools(QtWidgets.QWidget):
         self._nav_data = self._visible_data()
         self._nav_viewers = {}
         for data in self._nav_data:
-            pix_cid = is_convertible_to_single_pixel_cid(data, self.viewer.state.x_att)
+            pix_cid = is_convertible_to_single_pixel_cid(data, self.viewer.state.x_att_pixel)
             self._nav_viewers[data] = self._viewers_with_data_slice(data, pix_cid)
 
     def _on_slider_change(self, *args):
@@ -167,7 +167,7 @@ class ProfileTools(QtWidgets.QWidget):
             axis = self.viewer.state.x_att.axis
             slc = int(round(x))
         else:
-            pix_cid = is_convertible_to_single_pixel_cid(data, self.viewer.state.x_att)
+            pix_cid = is_convertible_to_single_pixel_cid(data, self.viewer.state.x_att_pixel)
             axis = pix_cid.axis
             axis_view = [0] * data.ndim
             axis_view[pix_cid.axis] = slice(None)
@@ -215,7 +215,7 @@ class ProfileTools(QtWidgets.QWidget):
         def on_done():
             self.ui.button_fit.setText("Fit")
             self.ui.button_fit.setEnabled(True)
-            self.canvas.draw()
+            self.canvas.draw_idle()
 
         self.ui.button_fit.setText("Running...")
         self.ui.button_fit.setEnabled(False)
@@ -237,7 +237,7 @@ class ProfileTools(QtWidgets.QWidget):
     def _on_clear(self):
         self.ui.text_log.document().setPlainText('')
         self._clear_fit()
-        self.canvas.draw()
+        self.canvas.draw_idle()
 
     def _fit(self, fitter, xlim=None):
 
@@ -273,7 +273,7 @@ class ProfileTools(QtWidgets.QWidget):
                                                  color=layer.state.color,
                                                  normalize=normalize.get(layer, None))[0])
 
-        self.canvas.draw()
+        self.canvas.draw_idle()
 
     def _visible_data(self):
         datasets = set()
@@ -310,7 +310,7 @@ class ProfileTools(QtWidgets.QWidget):
 
         for data in self._visible_data():
 
-            pix_cid = is_convertible_to_single_pixel_cid(data, self.viewer.state.x_att)
+            pix_cid = is_convertible_to_single_pixel_cid(data, self.viewer.state.x_att_pixel)
 
             for viewer in self._viewers_with_data_slice(data, pix_cid):
 
@@ -325,6 +325,8 @@ class ProfileTools(QtWidgets.QWidget):
                 if isinstance(current_slice, AggregateSlice):
                     current_slice = current_slice.center
 
+                imin, imax = min(imin, imax), max(imin, imax)
+
                 slices[axis] = AggregateSlice(slice(imin, imax),
                                               current_slice,
                                               func)
