@@ -1,5 +1,6 @@
 from __future__ import absolute_import, division, print_function
 
+import warnings
 import numpy as np
 
 from matplotlib.colors import Normalize
@@ -10,7 +11,7 @@ from mpl_scatter_density.generic_density_artist import GenericDensityArtist
 from astropy.visualization import (ImageNormalize, LinearStretch, SqrtStretch,
                                    AsinhStretch, LogStretch)
 
-from glue.utils import defer_draw, broadcast_to, nanmax, categorical_ndarray
+from glue.utils import defer_draw, broadcast_to, nanmax, ensure_numerical
 from glue.viewers.scatter.state import ScatterLayerState
 from glue.viewers.scatter.python_export import python_export_scatter_layer
 from glue.viewers.matplotlib.layer_artist import MatplotlibLayerArtist
@@ -58,13 +59,6 @@ class DensityMapLimits(object):
         return 10. ** (np.log10(nanmax(array)) * self.contrast)
 
 
-def ensure_numerical(values):
-    if isinstance(values, categorical_ndarray):
-        return values.codes
-    else:
-        return values
-
-
 def set_mpl_artist_cmap(artist, values, state=None, cmap=None, vmin=None, vmax=None):
 
     if state is not None:
@@ -157,11 +151,13 @@ class ScatterLayerArtist(MatplotlibLayerArtist):
 
         # Scatter density
         self.density_auto_limits = DensityMapLimits()
-        self.density_artist = GenericDensityArtist(self.axes, color='white',
-                                                   vmin=self.density_auto_limits.min,
-                                                   vmax=self.density_auto_limits.max,
-                                                   update_while_panning=False,
-                                                   histogram2d_func=self.state.compute_density_map)
+        with warnings.catch_warnings():
+            warnings.filterwarnings("ignore", message='All-NaN slice encountered')
+            self.density_artist = GenericDensityArtist(self.axes, color='white',
+                                                       vmin=self.density_auto_limits.min,
+                                                       vmax=self.density_auto_limits.max,
+                                                       update_while_panning=False,
+                                                       histogram2d_func=self.compute_density_map)
         self.axes.add_artist(self.density_artist)
 
         self.mpl_artists = [self.scatter_artist, self.plot_artist,
@@ -170,14 +166,25 @@ class ScatterLayerArtist(MatplotlibLayerArtist):
         self.errorbar_index = 2
         self.vector_index = 3
 
-        self.reset_cache()
+        # NOTE: Matplotlib can't deal with NaN values in errorbar correctly, so
+        # we need to prefilter values - the following variable is used to store
+        # the mask for the values we keep, so that we can apply it to the color
+        # See also https://github.com/matplotlib/matplotlib/issues/13799
+        self._errorbar_keep = None
 
-    def reset_cache(self):
-        self._last_viewer_state = {}
-        self._last_layer_state = {}
+    def compute_density_map(self, *args, **kwargs):
+        try:
+            density_map = self.state.compute_density_map(*args, **kwargs)
+        except IncompatibleAttribute:
+            self.disable_invalid_attributes(self._viewer_state.x_att,
+                                            self._viewer_state.y_att)
+            return np.array([[np.nan]])
+        else:
+            self.enable()
+        return density_map
 
     @defer_draw
-    def _update_data(self, changed):
+    def _update_data(self):
 
         # Layer artist has been cleared already
         if len(self.mpl_artists) == 0:
@@ -286,17 +293,28 @@ class ScatterLayerArtist(MatplotlibLayerArtist):
 
         if self.state.xerr_visible or self.state.yerr_visible:
 
+            keep = ~np.isnan(x) & ~np.isnan(y)
+
             if self.state.xerr_visible and self.state.xerr_att is not None:
-                xerr = ensure_numerical(self.layer[self.state.xerr_att].ravel())
+                xerr = ensure_numerical(self.layer[self.state.xerr_att].ravel()).copy()
+                keep &= ~np.isnan(xerr)
             else:
                 xerr = None
 
             if self.state.yerr_visible and self.state.yerr_att is not None:
-                yerr = ensure_numerical(self.layer[self.state.yerr_att].ravel())
+                yerr = ensure_numerical(self.layer[self.state.yerr_att].ravel()).copy()
+                keep &= ~np.isnan(yerr)
             else:
                 yerr = None
 
-            self.errorbar_artist = self.axes.errorbar(x, y, fmt='none',
+            if xerr is not None:
+                xerr = xerr[keep]
+            if yerr is not None:
+                yerr = yerr[keep]
+
+            self._errorbar_keep = keep
+
+            self.errorbar_artist = self.axes.errorbar(x[keep], y[keep], fmt='none',
                                                       xerr=xerr, yerr=yerr)
             self.mpl_artists[self.errorbar_index] = self.errorbar_artist
 
@@ -429,7 +447,8 @@ class ScatterLayerArtist(MatplotlibLayerArtist):
                     if force or 'color' in changed or 'cmap_mode' in changed:
                         eartist.set_color(self.state.color)
                 elif force or any(prop in changed for prop in CMAP_PROPERTIES):
-                    c = ensure_numerical(self.layer[self.state.cmap_att].ravel())
+                    c = ensure_numerical(self.layer[self.state.cmap_att].ravel()).copy()
+                    c = c[self._errorbar_keep]
                     set_mpl_artist_cmap(eartist, c, self.state)
 
                 if force or 'alpha' in changed:
@@ -455,7 +474,15 @@ class ScatterLayerArtist(MatplotlibLayerArtist):
                 artist.set_zorder(self.state.zorder)
 
             if force or 'visible' in changed:
-                artist.set_visible(self.state.visible)
+                # We need to hide the density artist if it is not needed because
+                # otherwise it might still show even if there is no data as the
+                # neutral/zero color might not be white.
+                if artist is self.density_artist:
+                    artist.set_visible(self.state.visible and
+                                       self.state.density_map and
+                                       self.state.markers_visible)
+                else:
+                    artist.set_visible(self.state.visible)
 
         self.redraw()
 
@@ -467,30 +494,10 @@ class ScatterLayerArtist(MatplotlibLayerArtist):
                 self.state.layer is None):
             return
 
-        # Figure out which attributes are different from before. Ideally we shouldn't
-        # need this but currently this method is called multiple times if an
-        # attribute is changed due to x_att changing then hist_x_min, hist_x_max, etc.
-        # If we can solve this so that _update_histogram is really only called once
-        # then we could consider simplifying this. Until then, we manually keep track
-        # of which properties have changed.
-
-        changed = set()
-
-        if not force:
-
-            for key, value in self._viewer_state.as_dict().items():
-                if value != self._last_viewer_state.get(key, None):
-                    changed.add(key)
-
-            for key, value in self.state.as_dict().items():
-                if value != self._last_layer_state.get(key, None):
-                    changed.add(key)
-
-        self._last_viewer_state.update(self._viewer_state.as_dict())
-        self._last_layer_state.update(self.state.as_dict())
+        changed = set() if force else self.pop_changed_properties()
 
         if force or len(changed & DATA_PROPERTIES) > 0:
-            self._update_data(changed)
+            self._update_data()
             force = True
 
         if force or len(changed & VISUAL_PROPERTIES) > 0:
