@@ -17,12 +17,11 @@ from glue.core.roi import XRangeROI, RectangularROI, CircularROI
 from glue.core.subset import RoiSubsetState, AndState
 from glue import core
 from glue.core.component_id import ComponentID
-from glue.utils.qt import combo_as_string
+from glue.utils.qt import combo_as_string, process_events
 from glue.viewers.matplotlib.qt.tests.test_data_viewer import BaseTestMatplotlibDataViewer
 from glue.core.state import GlueUnSerializer
 from glue.app.qt.layer_tree_widget import LayerTreeWidget
 from glue.app.qt import GlueApplication
-from glue.tests.helpers import PYSIDE2_INSTALLED  # noqa
 
 from ..data_viewer import ScatterViewer
 
@@ -265,7 +264,6 @@ class TestScatterViewer(object):
         self.viewer.state.layers[0].points_mode = 'markers'
         self.viewer.state.layers[0].points_mode = 'density'
 
-    @pytest.mark.skipif('PYSIDE2_INSTALLED')
     @pytest.mark.parametrize('protocol', [0, 1])
     def test_session_back_compat(self, protocol):
 
@@ -326,6 +324,8 @@ class TestScatterViewer(object):
         assert viewer3.state.layers[1].visible
         assert not viewer3.state.layers[2].visible
 
+        ga.close()
+
     def test_session_line_back_compat(self):
 
         # Backward-compatibility for v0.11 files in which the line and scatter
@@ -360,6 +360,8 @@ class TestScatterViewer(object):
         assert not viewer1.state.layers[0].markers_visible
         assert viewer1.state.layers[0].line_visible
 
+        ga.close()
+
     def test_save_svg(self, tmpdir):
         # Regression test for a bug in AxesCache that caused SVG saving to
         # fail (because renderer.buffer_rgba did not exist)
@@ -493,6 +495,8 @@ class TestScatterViewer(object):
         self.viewer.add_data(self.data)
         self.viewer.state.x_att = self.data.id['z']
 
+        process_events()
+
         assert visible_xaxis_labels(self.viewer.axes) == ['a', 'b', 'c']
 
         self.session.application.save_session(filename)
@@ -510,6 +514,8 @@ class TestScatterViewer(object):
         assert viewer.state.x_att is dc[0].id['z']
         assert visible_xaxis_labels(self.viewer.axes) == ['a', 'b', 'c']
 
+        ga.close()
+
     def test_enable_disable_components_combo(self):
 
         # Regression test for a bug that caused an error when turning off pixel
@@ -590,3 +596,50 @@ class TestScatterViewer(object):
         assert options.valuetext_x_max.text() == '1971-02-17'
         assert options.valuetext_y_min.text() == '3.188'
         assert options.valuetext_y_max.text() == '3.512'
+
+        ga.close()
+
+    def test_density_map_incompatible_subset(self, capsys):
+
+        # Regression test for a bug that caused the scatter viewer to crash
+        # if subset for density map was incompatible.
+
+        data2 = Data(label='d1', x=[3.4, 2.3, -1.1, 0.3], y=[3.2, 3.3, 3.4, 3.5], z=['a', 'b', 'c', 'a'])
+
+        self.data_collection.append(data2)
+
+        self.viewer.add_data(self.data)
+        self.viewer.add_data(data2)
+
+        self.data_collection.new_subset_group('test', self.data.id['x'] > 1)
+
+        for layer in self.viewer.state.layers:
+            layer.density_map = True
+
+        self.viewer.figure.canvas.draw()
+        process_events()
+
+        assert self.viewer.layers[0].enabled
+        assert not self.viewer.layers[1].enabled
+        assert self.viewer.layers[2].enabled
+        assert not self.viewer.layers[3].enabled
+
+    def test_density_map_line_error_vector(self, capsys):
+
+        # Make sure that we don't allow/show lines/errors/vectors
+        # if in density map mode.
+
+        self.viewer.add_data(self.data)
+
+        self.viewer.state.layers[0].line_visible = True
+        self.viewer.state.layers[0].xerr_visible = True
+        self.viewer.state.layers[0].yerr_visible = True
+        self.viewer.state.layers[0].vector_visible = True
+
+        # Setting density_map to True resets the visibility of
+        # lines/errors/vectors.
+        self.viewer.state.layers[0].density_map = True
+        assert not self.viewer.state.layers[0].line_visible
+        assert not self.viewer.state.layers[0].xerr_visible
+        assert not self.viewer.state.layers[0].yerr_visible
+        assert not self.viewer.state.layers[0].vector_visible
